/*************************************************************************
 *                                                                          
 * axes.cpp  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#include "axes.h"
#include "util.h"

#include <qlabel.h>

Axes::Axes()
  : name("Axes1"),
    description("<No description available>"),
    color(Qt::red),
    definedPoints(0), scaleXAxis(0), scaleYAxis(0)
{}

Axes::~Axes() {}

QTextStream& operator<<(QTextStream & output, const Axes & aAxes)
{
  output << "    <axes name=\"" << aAxes.name << "\"\n"
         << "          description=\"" << aAxes.description << "\"\n"
         << "          npoints=\"" << aAxes.definedPoints
         << "\" xscale=\"" << aAxes.scaleXAxis
         << "\" yscale=\"" << aAxes.scaleYAxis
         << "\" color=\"" << aAxes.color.name() << "\">\n";

  int i;
  AxisPoint p;
  for (i=0; i<aAxes.definedPoints; i++) {
    p = aAxes.p[i];
    output << "      <point "
           << "gx=\"" << p.getGX()
           << "\" gy=\"" << p.getGY()
           << "\" x=\"" << p.getX()
           << "\" y=\"" << p.getY() << "\"/>\n";
  }

  output << "    </axes>\n\n";

  return output;
}

/* Defines a new point */
void Axes::newPoint(double x, double y, double gx, double gy)
{
  if (definedPoints < 3) {
    p[definedPoints].set( x, y, gx, gy );
    p[definedPoints].setCanvas(theCanvas);
    p[definedPoints].setZ(100.);
    p[definedPoints].setPen(color);
    //qDebug("{%s}.p[%d] = {%f,%f,%f,%f}", name.latin1(), definedPoints, x, y, gx, gy);
    definedPoints++;
    //qDebug("{%s} has now %d defined points", name.latin1(), definedPoints);
  }
}

/** removes point from the axes */
void Axes::removePoint(AxisPoint *ap)
{
  for (int i=0; i<definedPoints; i++) {
    if ( &(p[i]) == ap ) {
      for (int j=i+1; j<definedPoints; j++)
        p[j-1].set( p[j].getX(),
                    p[j].getY(),
                    p[j].getGX(),
                    p[j].getGY() );

      definedPoints--;
      break;
    }
  }
}

/** Returns if point belongs to the axes */
bool Axes::contains(AxisPoint *ap) {
  bool result = false;
  for (int i=0; i<definedPoints; i++)
    result = result || ( &(p[i]) == ap );

  return result;
}

/** Returns reference to i-th axis point */
AxisPoint* const Axes::getPoint(int i)
{
  if (i<definedPoints)
    return &(p[i]);
  else
    return 0;
}

/** Returns X Axis Scale selected by the user */
int Axes::getXAxisScale()
{
  return scaleXAxis;
}

/** Returns Y Axis Scale selected by the user */
int Axes::getYAxisScale()
{
  return scaleYAxis;
}

/** Sets scales for both axis */
void Axes::setScales(int sx,int sy)
{
  scaleXAxis = sx;
  scaleYAxis = sy;
}

/** Propagate colors */
void Axes::propagateColor()
{
  int i;
  for ( i=0; i<definedPoints; i++ ) {
    p[i].setPen( color );
  }
}
