/*************************************************************************
 *                                                                          
 * axes.h  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#ifndef AXES_H
#define AXES_H

/**
  *@author J C Gonzalez
  */

#include "axispoint.h"

#include <qtextstream.h>

#define AXES_TYPE_LINEAR       0
#define AXES_TYPE_LOGARITHMIC  1


class Axes {

  friend QTextStream& operator<<(QTextStream &, const Axes &);
//  friend QTextStream& operator>>(QTextStream &, Axes &);

public:
  Axes();
  ~Axes();

  inline void setCanvas(QCanvas* c) { theCanvas = c; }

  inline void setName(QString& s)     { name = s; }
  inline QString& getName(void)       { return name; }

  inline void setDesc(QString& s)     { description = s; }
  inline QString& getDesc(void)       { return description; }

  inline void setColor(QColor& c)     { color = c; }
  inline QColor getColor(void) const  { return color; }

  inline int getNPoints(void) const   { return definedPoints; }

  /** Returns TRUE if all the points (3) are already defined */
  inline bool complete() const { return (definedPoints == 3); }

  /** Defines a new point */
  void newPoint(double x, double y, double gx, double gy);
  /** removes point from the axes */
  void removePoint(AxisPoint *ap);
  /** Returns if point belongs to the axes */
  bool contains(AxisPoint *ap);

  /** Returns reference to i-th axis point */
  AxisPoint* const getPoint(int i);
  /** Returns X Axis Scale selected by the user */
  int getXAxisScale();
  /** Returns X Axis Scale selected by the user */
  int getYAxisScale();
  /** Sets scales for both axis */
  void setScales(int,int);
  /** Propagate colors */
  void propagateColor();

private:
  QString       name;
  QString       description;
  AxisPoint     p[3];
  QCanvas       *theCanvas;
  QColor        color;
  int           definedPoints;
  int           scaleXAxis;
  int           scaleYAxis;
};


#endif
