/*************************************************************************
 *                                                                          
 * curve.cpp  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#include "curve.h"
#include "util.h"

#include <qlabel.h>

#include <qmap.h>
#include <qstring.h>

extern QMap <int, QString> Plugins_List;

Curve::Curve()
  : name("Curve1"),
    description("<No description available>"),
    color(Qt::red),
    theModel(MODEL_X_Y, 0, 0)
{
  pList = new PointList;
  pList->setAutoDelete( TRUE );
  pSortedList = 0;
}

Curve::~Curve()
{
  delete pList;
}

QTextStream& operator<<(QTextStream & output, const Curve & aCurve)
{
  Model model = aCurve.theModel;

  output << "    <curve name=\"" << aCurve.name << "\"\n"
         << "           description=\"" << aCurve.description << "\"\n"
         << "           npoints=\"" << aCurve.pList->count()  << "\"\n"
         << "           model=\"" << model.getModel()
                        << "\" fx=\"" << model.getFx() << "\" fy=\"" << model.getFy() << "\"\n"
         << "           color=\"" << aCurve.color.name() << "\">\n";

  PointListIterator it(*(aCurve.pList)); // iterator for list
  CurvePoint *p;
  for ( ; it.current(); ++it ) {
    p = it.current();
    output << "      <point "
           << "gx=\"" << p->getGX()
           << "\" gy=\"" << p->getGY() << "\"/>\n";
  }

  output << "    </curve>\n\n";

  return output;
}

/** removes point from the curve */
void Curve::removePoint(CurvePoint *cp)
{
  if (pList->removeRef( cp )) qDebug("REMOVED!");
}

/** Returns if point belongs to the curve */
bool Curve::contains(CurvePoint *cp) {
  return (pList->containsRef( cp ) > 0);
}

/** Defines a new point */
void Curve::newPoint(double x, double y, double gx, double gy)
{
  CurvePoint* p = new CurvePoint(theCanvas);
  p->set( x, y, gx, gy );
  p->setZ(200.);
  p->setPen(color);
  p->show();

  pList->append( p );
}

/** Propagate colors */
void Curve::propagateColor()
{
  PointListIterator it(*pList); // iterator for list
  for ( ; it.current(); ++it ) {
    it.current()->setPen( color );
  }
}

PointList* Curve::getPointSortedList(void)
{
  if (pSortedList != 0)
    qFatal("Memory leak: pSortedList should be 0!");

  pSortedList = new PointList(*pList);
  pSortedList->sort();

  return pSortedList;
}

void Curve::resetPointSortedList(void)
{
  if (pSortedList != 0) {
    delete pSortedList;
    pSortedList = 0;
  }
}

/** Read property of Model theModel. */
const Model& Curve::getModel(){
	return theModel;
}
