/*************************************************************************
 *                                                                          
 * curve.h  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#ifndef CURVE_H
#define CURVE_H

/**
  *@author J C Gonzalez
  */

#include <qtextstream.h>
#include <qcolor.h>

#include "pointlist.h"
#include "model.h"

class Curve {

  friend QTextStream& operator<<(QTextStream &, const Curve &);
  //friend QTextStream& operator>>(QTextStream &, Curve &);

public:
  Curve();
  virtual ~Curve();

  inline void setCanvas(QCanvas* c) { theCanvas = c; }

  inline void setName(QString& s)     { name = s; }
  inline QString getName(void) const  { return name; }

  inline void setDesc(QString& s)     { description = s; }
  inline QString getDesc(void) const  { return description; }

  inline void setColor(QColor& c)     { color = c; }
  inline QColor getColor(void) const  { return color; }

  inline PointList* getPointList(void) const { return pList; }

  inline int getNPoints(void)         { return pList->count(); }

  PointList* getPointSortedList(void);
  void       resetPointSortedList(void);

  /** Defines a new point */
  void newPoint(double x, double y, double gx, double gy);

  /** removes point from the curve */
  void removePoint(CurvePoint *cp);

  /** Returns if point belongs to the curve */
  bool contains(CurvePoint *cp);

  /** Propagate colors */
  void propagateColor();

  /** Returns TRUE if this curve is a curve to be defined by the "Follow Curve Algorithm" */
  bool isFollowCurve() const { return followCurve; }

  /** Sets the "followCurve" status */
  void setFollowCurve(bool s) { followCurve = s; }

  /** Read property of Model theModel. */
  const Model& getModel();

private:
  PointList     *pList;
  PointList     *pSortedList;
  QString       name;
  QString       description;
  QCanvas       *theCanvas;
  QColor        color;
  bool          followCurve;

  /** Information about the model to be used when exporting data from a curve */
  Model         theModel;
};


#endif
