/*************************************************************************
 *                                                                          
 * deditelement.cpp  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#include "deditelement.h"
#include "dnewelement.h"
#include "dgrid.h"
#include "axes.h"
#include "curve.h"
#include "qscanplotdoc.h"

#include <qlistview.h>
#include <qmessagebox.h>

dEditElement::dEditElement(QWidget *parent, const char *name, bool modal )
  : dlgEditElement(parent,name,modal)
{
  pixmap.resize(32,16);
}

dEditElement::~dEditElement() {}

void dEditElement::setMode(EditMode m)
{
  int i;
  mode = m;
  switch (mode) {
  case EditAxes:  // remove from the Xmin, 4 columns
    for (i=0; i<4; i++)
      theList->removeColumn(5);
    break;
  case EditCurve: // remove from the Xscale, 6 columns
    for (i=0; i<6; i++)
      theList->removeColumn(3);
    break;
  case EditGrid: // remove from the Color, 3 columns
    for (i=0; i<3; i++)
      theList->removeColumn(2);
    break;
  default:
    break;
  }
}

void dEditElement::setDoc(QScanPlotDoc* d)
{
  doc = d;
}

void dEditElement::addItem(const QString& name, const QString& desc,
                           const QColor& c, int xscale, int yscale,
                           double x1, double x2, int n, double dx)
{
  QListViewItem * item = new QListViewItem( theList );
  item->setText( Name, name );
  item->setText( Description, desc );

  if (mode != EditGrid) {
    pixmap.fill( c );
    item->setPixmap( Color, pixmap );
  }
  
  if (mode == EditAxes) {
    item->setText( Xscale, ((xscale == 0) ? "Linear" : "Logarithmic"));
    item->setText( Yscale, ((yscale == 0) ? "Linear" : "Logarithmic"));
  } else if (mode == EditGrid) {
    QString s;
    item->setText( Xmin, s.setNum(x1));
    item->setText( Xmax, s.setNum(x2));
    item->setText( Intervals, s.setNum(n));
    item->setText( IntWidth, s.setNum(dx));
  }
}

void dEditElement::modifyItem(QListViewItem * item,
                              const QString& name, const QString& desc,
                              const QColor& c, int xscale, int yscale,
                              double x1, double x2, int n, double dx)
{
  item->setText( Name, name );
  item->setText( Description, desc );

  if (mode != EditGrid) {
    pixmap.fill( c );
    item->setPixmap( Color, pixmap );
  }
  
  if (mode == EditAxes) {
    item->setText( Xscale, ((xscale == 0) ? "Linear" : "Logarithmic"));
    item->setText( Yscale, ((yscale == 0) ? "Linear" : "Logarithmic"));
  } else if (mode == EditGrid) {
    QString s;
    item->setText( Xmin, s.setNum(x1));
    item->setText( Xmax, s.setNum(x2));
    item->setText( Intervals, s.setNum(n));
    item->setText( IntWidth, s.setNum(dx));
  }
}

void dEditElement::slotEditElement()
{
  slotEditElement( theList->selectedItem() );
}

void dEditElement::slotEditElement(QListViewItem* it)
{
  dNewElement dlg;

  switch (mode) {

    case EditAxes:
    {
      Axes *a;
      QString name(it->text(0));
      int i;
      for (i=0; i<doc->getNAxes(); i++){
        a = doc->getAxes(i);
        if (name == a->getName()) break;
      }
      if (i == doc->getNAxes())
        qFatal("Something strange happens here. Couldn't find this axes.");

      QColor  col(a->getColor());
      QString desc(a->getDesc());
      dlg.setTitle("Edit Axes System");
      dlg.setBtnColor(col);
      dlg.setName(name);
      dlg.setDesc(desc);
      dlg.setScales(a->getXAxisScale(), a->getYAxisScale());
      if ( dlg.exec() ) {
        name = dlg.getName();
        desc = dlg.getDesc();
        col  = dlg.getColor();
        a->setName( name );
        a->setDesc( desc );
        a->setColor( col );
        a->setScales(dlg.getXAxisScale(), dlg.getYAxisScale());
        modifyItem(it, name, desc, col, dlg.getXAxisScale(), dlg.getYAxisScale(), 0., 0., 0, 0.);
        a->propagateColor();
      }
      break;
    }
    case EditCurve:
    {
      Curve *c;
      QString name(it->text(0));
      int i;
      for (i=0; i<doc->getNCurves(); i++){
        c = doc->getCurve(i);
        if (name == c->getName()) break;
      }
      if (i == doc->getNCurves())
        qFatal("Something strange happens here. Couldn't find this axes.");

      QColor  col(c->getColor());
      QString desc(c->getDesc());
      dlg.setTitle("Edit Curve");
      dlg.setBtnColor(col);
      dlg.setName(name);
      dlg.setDesc(desc);
      dlg.hideAxesScales();
      if ( dlg.exec() ) {
        name = dlg.getName();
        desc = dlg.getDesc();
        col  = dlg.getColor();
        c->setName( name );
        c->setDesc( desc );
        c->setColor( col );
        modifyItem(it, name, desc, col, 0, 0, 0., 0., 0, 0.);
        c->propagateColor();
      }
      break;
    }
    case EditGrid:
    {
      dGrid gdlg;
      Grid *g;
      QString name(it->text(0));
      int i;
      for (i=0; i<doc->getNGrids(); i++){
        g = doc->getGrid(i);
        if (name == g->getName()) break;
      }
      if (i == doc->getNGrids())
        qFatal("Something strange happens here. Couldn't find this grid.");

      QString desc(g->getDesc());
      gdlg.setName(name);
      gdlg.setDesc(desc);
      gdlg.init( g->getXmin(), g->getXmax(), g->getNIntervals(), g->getIntervalWidth() );
      if ( gdlg.exec() ) {
        name = gdlg.getName();
        desc = gdlg.getDesc();
        g->setName( name );
        g->setDesc( desc );
        g->init(gdlg.getXmin(), gdlg.getXmax(), gdlg.getNIntervals() );
        modifyItem(it, name, desc, Qt::black, 0, 0,
                   g->getXmin(), g->getXmax(), g->getNIntervals(), g->getIntervalWidth());
      }
      break;
    }
    default:
    {
      break;
    }

  }
}

void dEditElement::slotDeleteElement()
{

  QListViewItem* it = theList->selectedItem();

  switch (mode) {

    case EditAxes:
    {
      Axes *a;
      QString name(it->text(0));
      int i;
      for (i=0; i<doc->getNAxes(); i++){
        a = doc->getAxes(i);
        if (name == a->getName()) break;
      }
      if (i == doc->getNAxes())
        qFatal("Something strange happens here. Couldn't find this axes.");

      int answer = QMessageBox::critical(0, "Deleting Axes...",
                            "Are you really sure you want to delete this axes system?\n"
                            "This will remove *permanently* the Axes with all the information.\n\n"
                            "No, no, really, think about it.  Take your time...",
                            QMessageBox::Yes, QMessageBox::No | QMessageBox::Default | QMessageBox::Escape);
      if ( answer == QMessageBox::Yes ) {
        theList->takeItem( it );
        doc->deleteAxes( i );
      }
      break;
    }
    case EditCurve:
    {
      Curve *c;
      QString name(it->text(0));
      int i;
      for (i=0; i<doc->getNCurves(); i++){
        c = doc->getCurve(i);
        if (name == c->getName()) break;
      }
      if (i == doc->getNCurves())
        qFatal("Something strange happens here. Couldn't find this axes.");

      int answer = QMessageBox::critical(0, "Deleting Curve...",
                            "Are you really sure you want to delete this curve,\nwith all its points?\n"
                            "This will remove *permanently* the Curve with all the information.\n\n"
                            "No, no, really, think about it.  Take your time...",
                            QMessageBox::Yes, QMessageBox::No | QMessageBox::Default | QMessageBox::Escape);
      if ( answer == QMessageBox::Yes ) {
        theList->takeItem( it );
        doc->deleteCurve( i );
      }
      break;
    }
    case EditGrid:
    {
      dGrid gdlg;
      Grid *g;
      QString name(it->text(0));
      int i;
      for (i=0; i<doc->getNGrids(); i++){
        g = doc->getGrid(i);
        if (name == g->getName()) break;
      }
      if (i == doc->getNGrids())
        qFatal("Something strange happens here. Couldn't find this grid.");

      int answer = QMessageBox::critical(0, "Deleting Grid...",
                            "Are you really sure you want to delete this grid\n"
                            "This will remove *permanently* the Grid with all the information.\n\n"
                            "No, no, really, think about it.  Take your time...",
                            QMessageBox::Yes, QMessageBox::No | QMessageBox::Default | QMessageBox::Escape);
      if ( answer == QMessageBox::Yes ) {
        theList->takeItem( it );
        doc->deleteGrid( i );
      }
      break;
    }
    default:
    {
      break;
    }

  }
}

