/*************************************************************************
 *                                                                          
 * dexportsetup.cpp  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#include "dexportsetup.h"

#include <qlistbox.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <qmessagebox.h>

dExportSetup::dExportSetup(QWidget *parent, const char *name, bool modal)
  : dlgExportSetup(parent,name,modal)
{
  includedCurves->clear();
  excludedCurves->clear();

  init();
}

dExportSetup::~dExportSetup()
{
}

void dExportSetup::init()
{
  axes2use      = 0;
  grid2use      = 0;
  idCriteria    = X1Y1X2Y2;
  idSeparator   = Space;
  idComment     = Number;
  idNumeric     = Automatic;
  idLayout      = AllInOneLine;
  specialFormat = NoSpecial;
  updateGUI();
}

void dExportSetup::init(int axes, int grid, int crit, int lay, int sep,
                        int num, int com, int special)
{
  axes2use      = axes;
  grid2use      = grid;
  idCriteria    = crit;
  idSeparator   = sep;
  idComment     = com;
  idNumeric     = num;
  idLayout      = lay;
  specialFormat = special;
  updateGUI();
}

void dExportSetup::updateGUI()
{
  listAxes->setCurrentItem  ( axes2use );
  listGrids->setCurrentItem ( grid2use );
  groupCriteria->setButton  ( idCriteria );
  groupLayout->setButton    ( idLayout );
  groupComments->setButton  ( idComment );
  groupSeparators->setButton( idSeparator );
  groupNumeric->setButton   ( idNumeric );
  groupSpecial->setButton   ( specialFormat );

  bool enabled = (specialFormat == NoSpecial);

  //groupLayout->setEnabled( enabled );
  groupComments->setEnabled( enabled );
  groupSeparators->setEnabled( enabled );

  update();
}

void dExportSetup::includeItem(QString s)
{
  includedCurves->insertItem(s);
}

void dExportSetup::excludeItem(QString s)
{
  excludedCurves->insertItem(s);
}

void dExportSetup::addAxesSystem(QString s)
{
  listAxes->insertItem(s);
}

void dExportSetup::clearAxesSystem()
{
  listAxes->clear();
}

void dExportSetup::setAxesSystem(int i)
{
  listAxes->setCurrentItem(i);
}

int dExportSetup::getAxesSystem()
{
  return listAxes->currentItem();
}

void dExportSetup::addGrid(QString s)
{
  listGrids->insertItem(s);
}

void dExportSetup::clearGrid()
{
  listGrids->clear();
}

void dExportSetup::setGrid(int i)
{
  listGrids->setCurrentItem(i);
}

int dExportSetup::getGrid()
{
  return listGrids->currentItem();
}

void dExportSetup::slotExcludeCurve()
{
  int i = includedCurves->currentItem();
  if (i < 0) return;
  excludedCurves->insertItem( includedCurves->currentText() );
  includedCurves->removeItem( i );
}

void dExportSetup::slotIncludeCurve()
{
  int i = excludedCurves->currentItem();
  if (i < 0) return;
  includedCurves->insertItem( excludedCurves->currentText() );
  excludedCurves->removeItem( i );
}

void dExportSetup::slotHelp()
{
  QMessageBox::information(this, "Help",
                                 "Sorry, this item is still not available.",
                                  QMessageBox::Ok);
}

int dExportSetup::getNIncluded()
{
  return includedCurves->count();
}

QString dExportSetup::getIncluded(int i) const
{
  return includedCurves->text(i);
}

int dExportSetup::getNExcluded()
{
  return excludedCurves->count();
}

QString dExportSetup::getExcluded(int i) const
{
  return excludedCurves->text(i);
}

void dExportSetup::slotChangeAxes(int i)
{
  axes2use = i;
}

void dExportSetup::slotChangeGrid(int i)
{
  grid2use = i;
}

void dExportSetup::slotChangeSeparator(int i)
{
  idSeparator = i;
}

void dExportSetup::slotChangeComment(int i)
{
  idComment = i;
}

void dExportSetup::slotChangeNumeric(int i)
{
  idNumeric = i;
}

void dExportSetup::slotChangeLayout(int i)
{
  idLayout = i;
}

void dExportSetup::slotChangeCriteria(int i)
{
  idCriteria = i;
}

void dExportSetup::slotSetSpecialFormat(int i)
{
  specialFormat = i;
  bool enabled = (specialFormat == NoSpecial);
  //groupLayout->setEnabled( enabled );
  groupComments->setEnabled( enabled );
  groupSeparators->setEnabled( enabled );
}

QString dExportSetup::getCommentString() const
{
  switch (idComment) {
  case None:
    return " ";
    break;
  case Number:
    return "# ";
    break;
  case Cpp:
    return "// ";
    break;
  case Percent:
    return "% ";
    break;
  case Other:
    return otherComment->text();
    break;
  default:
    return "# ";
  }
}

QString dExportSetup::getUserFormatString() const
{
  return userFormat->text();
}

QString dExportSetup::getSeparatorString() const
{
  switch (idSeparator) {
  case Space:
    return " ";
    break;
  case Comma:
    return ",";
    break;
  case Tab:
    return "\t";
    break;
  case Semicolon:
    return ";";
    break;
  case OtherSeparator:
    return otherSep->text();
    break;
  default:
    return " ";
  }
}

int dExportSetup::getComment()
{
  return idComment;
}

int dExportSetup::getSeparator()
{
  return idSeparator;
}

int dExportSetup::getNumeric()
{
  return idNumeric;
}

int dExportSetup::getLayout()
{
  return idLayout;
}

int dExportSetup::getCriteria()
{
  return idCriteria;
}

int dExportSetup::getSpecialFormat()
{
  return specialFormat;
}






