/*************************************************************************
 *                                                                          
 * dgrid.cpp  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#include "dgrid.h"

#include <math.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qslider.h>

dGrid::dGrid(QWidget *parent, const char *name, bool modal )
  : dlgGrid(parent,name, modal), modifying(false) {}

dGrid::~dGrid() {}

/** Puts initial values */
void dGrid::init(double x1, double x2, int n, double dx)
{
  xmin = x1;
  xmax = x2;
  nbins = n;
  incx = dx;
  domain = x2 - x1;

  QString s;
  txtXmin->setText(s.setNum(xmin));
  txtXmax->setText(s.setNum(xmax));
  spNIntervals->setValue(nbins);
  txtIntWidth->setText(s.setNum(incx, 'g', 4));
}


/** Returns name entered by the user */
QString dGrid::getName() const
{
  return txtName->text();
}

/** Returns description entered by the user */
QString dGrid::getDesc() const
{
  return txtDesc->text();
}

/** Sets initial name */
void dGrid::setName(const QString& s)
{
  txtName->setText(s);
}

/** Sets initial description */
void dGrid::setDesc(const QString& s)
{
  txtDesc->setText(s);
}

#define CALC_INTERVALWIDTH_FROM_INDEX(p)  double( domain / pow(10., (2. - (p / 100.))) )
#define CALC_INDEX_FROM_INTERVALWIDTH(x)  int( (2. - log10(domain / x)) * 100. )

void dGrid::slotChangedIntervalWidth(const QString& s)
{
  if (! modifying) {
    modifying = true;
    incx = s.toDouble();
    nbins = int(domain / incx);
    spNIntervals->setValue( nbins );
  }
  modifying = false;
}

void dGrid::slotChangedNIntervals(int n)
{
  if (! modifying) {
    modifying = true;
    nbins = n;
    incx = domain / nbins;
    QString s;
    txtIntWidth->setText(s.setNum(incx, 'g', 4));
  }
  modifying = false;
}

void dGrid::slotChangedXmax(const QString& s)
{
  xmax = s.toDouble();
  domain = xmax - xmin;
  slotChangedNIntervals(nbins);
}

void dGrid::slotChangedXmin(const QString& s)
{
  xmin = s.toDouble();
  domain = xmax - xmin;
  slotChangedNIntervals(nbins);
}

void dGrid::setXmax(double x)
{
  xmax = x;
  QString s;
  txtXmax->setText(s.setNum(x));
  domain = xmax - xmin;
  slotChangedNIntervals(nbins);
}

void dGrid::setXmin(double x)
{
  xmin = x;
  QString s;
  txtXmin->setText(s.setNum(x));
  domain = xmax - xmin;
  slotChangedNIntervals(nbins);
}

void dGrid::slotAccept()
{
  accept();
}


