/*************************************************************************
 *                                                                          
 * dmodels.cpp  -  description
 *
 * Copyright (C) 2002  J C Gonzalez
 * jcgonzalez@rt.nl
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/

#include "dmodels.h"
#include <qlistbox.h>

#include <iostream>

dModels::dModels(QWidget *parent, const char *name, bool modal)
  : dlgModels(parent, name, modal), curveModels(0), nCurve(-1)
{
  lstCurves->clear();
  lstFx->clear();
  lstFy->clear();

  lstModel->setEnabled( false );
  lstFx->setEnabled( false );
  lstFy->setEnabled( false );
}

dModels::~dModels()
{
  if (curveModels != 0)
    delete [] curveModels;
}

void dModels::createModels(int n)
{
  if (curveModels == 0) {
    curveModels = new CurveModel[n];
    numCurveModels = n;
  }
}

void dModels::setCurveModel(int n, QString s, int m, int fx, int fy)
{
  if ( (curveModels != 0) && (n < numCurveModels) ) {
    curveModels[n].setModel(m);
    curveModels[n].setFx(fx);
    curveModels[n].setFy(fy);
    lstCurves->insertItem(s);
  }
}

int dModels::getCurveModel(int n)
{
  if ( (curveModels != 0) && (n < numCurveModels) )
    return curveModels[n].getModel();
  else
    return -1;
}

int dModels::getCurveModelFx(int n)
{
  if ( (curveModels != 0) && (n < numCurveModels) )
    return curveModels[n].getFx();
  else
    return -1;
}

int dModels::getCurveModelFy(int n)
{
  if ( (curveModels != 0) && (n < numCurveModels) )
    return curveModels[n].getFy();
  else
    return -1;
}

/** No descriptions */
void dModels::setFunctions(QMap<int,QString> lstPlugins)
{
  int i=0;
  int numPlg = int(lstPlugins.count());
  for (i=0; i<numPlg; i++) {
    lstFx->insertItem(lstPlugins[i]);
    lstFy->insertItem(lstPlugins[i]);
  }
}

void dModels::slotSelectCurve(int n)
{
  if (nCurve < 0) {
    lstModel->setEnabled( true );
    lstFx->setEnabled( true );
    lstFy->setEnabled( true );
  }

  nCurve = n;

  lstCurves->setCurrentItem( nCurve );
  lstModel->setCurrentItem( curveModels[nCurve].getModel() );
  lstFx->setCurrentItem( curveModels[nCurve].getFx() );
  lstFy->setCurrentItem( curveModels[nCurve].getFy() );
}

void dModels::slotSelectModel(int n)
{
  curveModels[nCurve].setModel( n );
}

void dModels::slotSelectFx(int n)
{
  curveModels[nCurve].setFx( n );
}

void dModels::slotSelectFy(int n)
{
  curveModels[nCurve].setFy( n );
}

