/*************************************************************************
 *                                                                          
 * dnewelement.cpp  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#include "dnewelement.h"

#include <qlabel.h>
#include <qlineedit.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qcolordialog.h>

dNewElement::dNewElement(QWidget *parent, const char *name, bool modal )
  : dlgNewElement(parent,name,modal), scaleXAxis(0), scaleYAxis(0) {}

dNewElement::~dNewElement() {}

/** Sets title for dialog */
void dNewElement::setTitle(const QString & s)
{
  lblTitle->setText(s);
}

/** Returns name entered by the user */
QString dNewElement::getName() const
{
  return txtName->text();
}

/** Returns description entered by the user */
QString dNewElement::getDesc() const
{
  return txtDesc->text();
}

/** Sets initial name */
void dNewElement::setName(const QString& s)
{
  txtName->setText(s);
}

/** Sets initial description */
void dNewElement::setDesc(const QString& s)
{
  txtDesc->setText(s);
}

/** Hide button groups in the dialog (for adding curves) */
void dNewElement::hideAxesScales()
{
  bgrpXAxis->hide();
  bgrpYAxis->hide();
}

/** Returns X Axis Scale selected by the user */
int dNewElement::getXAxisScale()
{
  return scaleXAxis;
}

/** Returns Y Axis Scale selected by the user */
int dNewElement::getYAxisScale()
{
  return scaleYAxis;
}

/** Sets initial scales */
void dNewElement::setScales(int sx, int sy)
{
  bgrpXAxis->setButton(sx);
  bgrpYAxis->setButton(sy);
}

void dNewElement::slotChangeXAxisScale(int n)
{
  scaleXAxis = n;
}

void dNewElement::slotChangeYAxisScale(int n)
{
  scaleYAxis = n;
}

/** Sets color for color selection button */
void dNewElement::setBtnColor(QColor &c)
{
  QPalette p = btnColor->palette();
  p.setColor(QColorGroup::Button, c );
  btnColor->setPalette(p);
}

/** Returns selected color */
QColor dNewElement::getColor() const
{
  QPalette p = btnColor->palette();
  return p.color(QPalette::Active, QColorGroup::Button);
}

/** Opens Color Dialog to select a color */
void dNewElement::slotSelectColor()
{
  QPalette p = btnColor->palette();
  QColor oldColor = p.color(QPalette::Active, QColorGroup::Button);
  QColor newColor = QColorDialog::getColor( oldColor, this );
  p.setColor(QColorGroup::Button, newColor );
  btnColor->setPalette(p);
}

