/*************************************************************************
 *                                                                          
 * grid.cpp  -  description
 *
 * Copyright (C) 2001 J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/

#include "grid.h"

// Implementation of class Grid

/** Constructor */
Grid::Grid() {}

/** Overloaded constructor */
Grid::Grid(Grid::GridValue x1, Grid::GridValue x2, Grid::GridValue ix)
{
  init(x1, x2, ix);
}

/** Overloaded constructor */
Grid::Grid(Grid::GridValue x1, Grid::GridValue x2, int n)
{
  init(x1, x2, n);
}

/** Copy Constructor */
//Grid::Grid(const Grid&) {}

/** Assignment */
//Grid& Grid::operator=(const Grid&) {}

/** Destructor */
Grid::~Grid() {}

QTextStream& operator<<(QTextStream & output, const Grid & aGrid)
{
  output << "    <grid name=\"" << aGrid.name << "\"\n"
         << "          description=\"" << aGrid.description << "\"\n"
         << "          xmin=\"" << aGrid.xmin
         << "\" xmax=\"" << aGrid.xmax
         << "\" intervals=\"" << aGrid.nbins
         << "\" step=\"" << aGrid.incx << "\"/>\n\n";

  return output;
}

/** Initializes grid internal values */
void Grid::init(Grid::GridValue x1, Grid::GridValue x2, Grid::GridValue ix)
{
  xmin  = x1;
  xmax  = x2;
  incx  = ix;
  nbins = int((x2 - x1) / ix);
}

/** Initializes grid internal values */
void Grid::init(Grid::GridValue x1, Grid::GridValue x2, int n)
{
  xmin  = x1;
  xmax  = x2;
  incx  = Grid::GridValue((x2 - x1) / n);
  nbins = n;
}

/** Resets counter */
void Grid::reset()
{
  icurrent = 0;
  xcurrent = xmin;
}

/** Returns first value in grid */
Grid::GridValue Grid::begin()
{
  return xmin;
}

/** Returns first value in grid */
Grid::GridValue Grid::first()
{
  return xmin;
}

/** Returns last value in grid */
Grid::GridValue Grid::end()
{
  return xmax;
}

/** Returns last value in grid */
Grid::GridValue Grid::last()
{
  return xmax;
}

/** Returns true if still in loop */
bool Grid::continues()
{
  return (icurrent < (nbins + 1));
}

/** Makes an step in the loop */
void Grid::step()
{
  icurrent++;
  xcurrent += incx;
}

/** Makes an step in the loop */
Grid::GridValue Grid::current()
{
  return xcurrent;
}

/** Makes an step and returns the next value */
Grid::GridValue Grid::next()
{
  step();
  return xcurrent;
}
