/*************************************************************************
 *                                                                          
 * grid.h  -  description
 *
 * Copyright (C) 2001 J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/

#ifndef GRID_H
#define GRID_H

/**
  *@author J C Gonzalez
  */

#include <qtextstream.h>

class Grid {
  
  friend QTextStream& operator<<(QTextStream &, const Grid &);

 public:
  
  typedef double GridValue;

  Grid(); // constructor
  Grid(GridValue x1, GridValue x2, GridValue ix); // constructor
  Grid(GridValue x1, GridValue x2, int n); // constructor

  //Grid(const Grid&);             // copy constructor
  //Grid& operator=(const Grid&);  // assignment

  ~Grid(); // destructor

public:

  /** Initializes grid internal values */
  void init(GridValue x1, GridValue x2, GridValue ix);
  /** Initializes grid internal values */
  void init(GridValue x1, GridValue x2, int n);

  inline void setName(QString& s)     { name = s; }
  inline QString& getName(void)       { return name; }

  inline void setDesc(QString& s)     { description = s; }
  inline QString& getDesc(void)       { return description; }

  inline GridValue getXmin(void) const          { return xmin; }
  inline GridValue getXmax(void) const          { return xmax; }
  inline GridValue getIntervalWidth(void) const { return incx; }
  inline int       getNIntervals(void) const    { return nbins; }
  inline int       getNPoints(void) const       { return nbins+1; }

  /** Resets counter */
  void reset();
  /** Returns first value in grid */
  GridValue begin();
  /** Returns first value in grid */
  GridValue first();
  /** Returns last value in grid */
  GridValue end();
  /** Returns last value in grid */
  GridValue last();
  /** Returns true if still in loop */
  bool continues();
  /** Makes an step in the loop */
  void step();
  /** Makes an step in the loop */
  GridValue current();
  /** Makes an step and returns the next value */
  GridValue next();

private:
  QString   name;
  QString   description;
  GridValue xmin;
  GridValue xmax;
  GridValue incx;
  int       nbins;

  int       icurrent;
  GridValue xcurrent;
  
};

#endif // ! GRID_H

