/*************************************************************************
 *                                                                          
 * lagrangeinterpolator.cpp  -  description
 *
 * Copyright (C) 2001 J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/

#include "lagrangeinterpolator2nd.h"

// Implementation of class LagrangeInterpolator2nd

/** Constructor */
LagrangeInterpolator2nd::LagrangeInterpolator2nd(double *px, double *py, int n)
  : LagrangeInterpolator(px,py,n) {}

QString LagrangeInterpolator2nd::description() const
{
  return QString(
  "Lagrange interpolator of 2nd. order,\n"
  "with extension for extrapolation of both ends\n"
  "of data set.");
}

//++
// Formula for Lagrange interpolation of 2nd order
// x: table(N): abscissas
// y: table(N): ordinates
// n: higher value of abscissas, such that t[0][n] <= x
// x0: value to be interpolated
//--

#define Lagrange(x,y,n,x0) \
              ((y[ (n) ]*((x0-x[(n)+1])*(x0-x[(n)+2]))/ \
                ((x[ (n) ]-x[(n)+1])*(x[ (n) ]-x[(n)+2])))+ \
               (y[(n)+1]*((x0-x[ (n) ])*(x0-x[(n)+2]))/ \
                ((x[(n)+1]-x[ (n) ])*(x[(n)+1]-x[(n)+2])))+ \
               (y[(n)+2]*((x0-x[ (n) ])*(x0-x[(n)+1]))/ \
                ((x[(n)+2]-x[ (n) ])*(x[(n)+2]-x[(n)+1]))) \
               )                             
  
//++
// Macro to find, and save in variable "m", the value of 
// "n" to be used in the "Lagrange(x,y,n,x0)" macro
//--

#define FindLagrange(x,y,m,maxm,x0) \
{\
  m = 0; \
  while ((x[++m] < x0) && (m < (maxm-2)));\
}\
--m
                
/** Performs actual interpolation */
double LagrangeInterpolator2nd::interpolate(double x0)
{
  register int iselected;

  FindLagrange(x,y,iselected,length,x0);
  return Lagrange(x,y,iselected,x0);
}
