/*************************************************************************
 *
 * qscanplot.cpp  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/

#include <stdlib.h>
#include <dlfcn.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>

#include "qscanplot.h"

#include <qdir.h>
#include <qsizepolicy.h>
#include <qmotifstyle.h>
#include <qmotifplusstyle.h>
#include <qwindowsstyle.h>
#include <qsgistyle.h>
#include <qplatinumstyle.h>
#include <qcdestyle.h>

#include <qdatetime.h>

#include "filenew.xpm"
#include "fileopen.xpm"
#include "fileimage.xpm"
#include "filesave.xpm"
#include "exportsetup.xpm"
#include "exportpreview.xpm"
#include "fileexport.xpm"
#include "axis.xpm"
#include "curve.xpm"
#include "grid.xpm"
#include "pointer.xpm"
#include "axispoint.xpm"
#include "curvepoint.xpm"
#include "recycled.xpm"

#include "qscanplot_image.xpm"

#include "dnewelement.h"
#include "deditelement.h"
#include "dgrid.h"
#include "dexportsetup.h"
#include "dexportpreview.h"
#include "dinterpolator.h"
#include "dmodels.h"
#include "dpreferences.h"
#include "dnewplugin.h"
#include "dabout.h"

#include "util.h"
#include "model.h"

extern QMap<int, QString> Plugins_List;
extern int numPlugins;

inline double normalized_zoom(int z) {
  return (double(z - ID_NORMALIZE_ZOOM) * .01);
}


QScanPlot::QScanPlot()
  : curvesList(0),
    axesList(0),
    dialogExportSetup(0),
    interpolator(0),
    hPlugins_Handle(0),
    pref(0)
{
  setIcon(QPixmap("qscanplot.xpm"));
  setCaption("QScanPlot " VERSION);

  
  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initMenuBar();
  initToolBar();
  initStatusBar();

  initDoc();
  initView();

  enableCommands();
}

QScanPlot::~QScanPlot()
{
}

void QScanPlot::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR

  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu

  fileMenu=new QPopupMenu();
  fileMenu->setCaption("File");
  fileMenu->insertTearOffHandle();
  fileMenu->insertItem(tr("&New"), this, SLOT(slotFileNew()), CTRL+Key_N, ID_FILE_NEW);
  fileMenu->insertItem("&Open...", this, SLOT(slotFileOpen()), CTRL+Key_O, ID_FILE_OPEN);
  fileMenu->insertItem("&Import Image...", this, SLOT(slotFileImportImage()), CTRL+Key_I, ID_FILE_IMPORT);
  fileMenu->insertSeparator();
  fileMenu->insertItem("&Save", this, SLOT(slotFileSave()), CTRL+Key_S, ID_FILE_SAVE);
  fileMenu->insertItem("Save &as...", this, SLOT(slotFileSaveAs()), 0, ID_FILE_SAVE_AS);
  fileMenu->insertItem("&Close", this, SLOT(slotFileClose()), CTRL+Key_W, ID_FILE_CLOSE);
  fileMenu->insertSeparator();
  fileMenu->insertItem("Export Set&up...", this, SLOT(slotFileExportSetup()), CTRL+Key_U, ID_FILE_EXPORTSETUP);
  fileMenu->insertItem("Export Pre&view...", this, SLOT(slotPreview()), CTRL+Key_V, ID_FILE_EXPORTPREVIEW);
  fileMenu->insertItem("&Export", this, SLOT(slotFileExport()), 0, ID_FILE_EXPORT);
  fileMenu->insertSeparator();
  fileMenu->insertItem("&Print", this, SLOT(slotFilePrint()), CTRL+Key_P, ID_FILE_PRINT);
  fileMenu->insertSeparator();
  fileMenu->insertItem("E&xit", this, SLOT(slotFileQuit()), CTRL+Key_Q, ID_FILE_QUIT);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry editMenu
  /*
  editMenu=new QPopupMenu();
  editMenu->setCaption("Edit");
  editMenu->insertTearOffHandle();
  editMenu->insertItem("Cu&t", this, SLOT(slotEditCut()), CTRL+Key_X, ID_EDIT_CUT);
  editMenu->insertItem("&Copy", this, SLOT(slotEditCopy()), CTRL+Key_C, ID_EDIT_COPY);
  editMenu->insertItem("&Paste", this, SLOT(slotEditPaste()), CTRL+Key_V, ID_EDIT_PASTE);
  */
  
  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  viewMenu=new QPopupMenu();
  viewMenu->setCaption("View");
  viewMenu->insertTearOffHandle();
  viewMenu->setCheckable(true);
  viewMenu->insertItem("Tool&bar", this, SLOT(slotViewToolBar()), 0, ID_VIEW_TOOLBAR);
  viewMenu->insertItem("&Statusbar", this, SLOT(slotViewStatusBar()), 0, ID_VIEW_STATUSBAR);
  viewMenu->insertSeparator();
  viewMenu->insertItem(" 1:4 ( 25%)", this, SLOT(slotChangeZoom(int)), 0, ID_VIEW_ZOOM25);
  viewMenu->insertItem(" 1:2 ( 50%)", this, SLOT(slotChangeZoom(int)), 0, ID_VIEW_ZOOM50);
  viewMenu->insertItem(" &1:1 (100%)", this, SLOT(slotChangeZoom(int)), 0, ID_VIEW_ZOOM100);
  viewMenu->insertItem(" &2:1 (200%)", this, SLOT(slotChangeZoom(int)), 0, ID_VIEW_ZOOM200);
  viewMenu->insertItem(" &4:1 (400%)", this, SLOT(slotChangeZoom(int)), 0, ID_VIEW_ZOOM400);

  viewMenu->setItemChecked(ID_VIEW_TOOLBAR, true);
  viewMenu->setItemChecked(ID_VIEW_STATUSBAR, true);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  elemMenu=new QPopupMenu();
  elemMenu->setCaption("Elements");
  elemMenu->insertTearOffHandle();
  //elemMenu->setCheckable(true);
  elemMenu->insertItem("New &Axes", this, SLOT(slotNewAxes()), Key_A, ID_ELEM_NEWAXES);
  elemMenu->insertItem("Edit A&xes", this, SLOT(slotEditAxes()), 0, ID_ELEM_EDITAXES);
  elemMenu->insertSeparator();
  elemMenu->insertItem("New &Curve", this, SLOT(slotNewCurve()), Key_C, ID_ELEM_NEWCURVE);
  elemMenu->insertItem("Edit C&urves", this, SLOT(slotEditCurves()), 0, ID_ELEM_EDITCURVES);
  elemMenu->insertSeparator();
  elemMenu->insertItem("New &Grid", this, SLOT(slotNewGrid()), Key_G, ID_ELEM_NEWGRID);
  elemMenu->insertItem("Edit G&rids", this, SLOT(slotEditGrids()), 0, ID_ELEM_EDITGRIDS);
  elemMenu->insertSeparator();
  elemMenu->insertItem("Edit &Models", this, SLOT(slotEditModels()), Key_M, ID_ELEM_EDITMODELS);
  elemMenu->insertSeparator();
  elemMenu->insertItem("Change to Pointer mode", this, SLOT(slotSelPointer()), 0, ID_ON_POINTER);
  elemMenu->insertItem("Change to Axis point mode", this, SLOT(slotSelAxisPoint()), 0, ID_ON_AXISPOINT);
  elemMenu->insertItem("Change to Curve point mode", this, SLOT(slotSelCurvePoint()), 0, ID_ON_CURVEPOINT);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry optMenu
  optMenu=new QPopupMenu();
  optMenu->setCaption("Options");
  optMenu->insertTearOffHandle();
  optMenu->setCheckable(true);
  optMenu->insertItem("&Sort curve points", this, SLOT(slotSetSort()), 0, ID_OPT_SORT);
  optMenu->insertItem("Select &Interpolator", this, SLOT(slotSetInterpolator()), 0, ID_OPT_INTERPOLATOR);
  optMenu->insertSeparator();
  optMenu->insertItem("&Preferences", this, SLOT(slotPreferences()), 0, ID_OPT_PREFERENCES);

  sortPoints = true;
  optMenu->setItemChecked(ID_OPT_SORT, true);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry toolMenu
  toolMenu=new QPopupMenu();
  toolMenu->setCaption("Tools");
  toolMenu->insertTearOffHandle();
  toolMenu->insertItem("Create Plug-in...", this, SLOT(slotCreatePlugin()), 0, ID_TOOL_CREATEPLUGIN);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  helpMenu=new QPopupMenu();
  helpMenu->insertItem("About...", this, SLOT(slotHelpAbout()), 0, ID_HELP_ABOUT);
  helpMenu->insertItem("About Qt...", this, SLOT(slotAboutQt()), 0, ID_HELP_ABOUT_QT);

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  // set menuBar() the current menuBar 

  menuBar()->insertItem("&File", fileMenu);
  /*menuBar()->insertItem("&Edit", editMenu);*/
  menuBar()->insertItem("&View", viewMenu);
  menuBar()->insertItem("E&lements", elemMenu);
  menuBar()->insertItem("&Options", optMenu);
  menuBar()->insertItem("&Tools", toolMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem("&Help", helpMenu);

  ///////////////////////////////////////////////////////////////////
  // CONNECT THE SUBMENU SLOTS WITH SIGNALS

  connect(fileMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  /*connect(editMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));*/
  connect(viewMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  connect(elemMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  connect(optMenu,  SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  connect(toolMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  connect(helpMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  
}

void QScanPlot::initToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  QPixmap openIcon, saveIcon, newIcon, imageIcon;
  QPixmap exportsetupIcon,exportIcon, previewIcon;
  QPixmap axisIcon, curveIcon, gridIcon;
  QPixmap pointerIcon, axisPointIcon, curvePointIcon, recycledIcon;

  fileToolbar = new QToolBar(this, "file operations");
 
  newIcon = QPixmap(filenew);
  QToolButton *fileNew = new QToolButton(newIcon, "New File", 0, this,
                                         SLOT(slotFileNew()), fileToolbar);

  openIcon = QPixmap(fileopen);
  QToolButton *fileOpen = new QToolButton(openIcon, "Open File", 0, this,
                                          SLOT(slotFileOpen()), fileToolbar);

  imageIcon = QPixmap(fileimage);
  QToolButton *fileImport = new QToolButton(imageIcon, "Import Image", 0, this,
                                          SLOT(slotFileImportImage()), fileToolbar);

  saveIcon = QPixmap(filesave);
  QToolButton *fileSave = new QToolButton(saveIcon, "Save File", 0, this,
                                          SLOT(slotFileSave()), fileToolbar);
  
  fileToolbar->addSeparator();

  exportsetupIcon = QPixmap(exportsetup);
  QToolButton *fileExportSetup = new QToolButton(exportsetupIcon, "Export Setup", 0, this,
                                                 SLOT(slotFileExportSetup()), fileToolbar);

  previewIcon = QPixmap(exportpreview);
  QToolButton *fileExportPreview = new QToolButton(previewIcon, "Export Preview", 0, this,
                                                   SLOT(slotPreview()), fileToolbar);

  exportIcon = QPixmap(fileexport);
  QToolButton *fileExport = new QToolButton(exportIcon, "Export Data", 0, this,
                                            SLOT(slotFileExport()), fileToolbar);

  fileToolbar->addSeparator();

  axisIcon = QPixmap(axis);
  QToolButton *fileAxis = new QToolButton(axisIcon, "Create new axes system", 0, this,
                                          SLOT(slotNewAxes()), fileToolbar);
  axesList = new QComboBox(fileToolbar);
  axesList->setEditable( FALSE );
  axesList->setMinimumWidth(140);

  fileToolbar->addSeparator();

  curveIcon = QPixmap(curve);
  QToolButton *fileCurve = new QToolButton(curveIcon, "Create new curve", 0, this,
                                           SLOT(slotNewCurve()), fileToolbar);
  curvesList = new QComboBox(fileToolbar);
  curvesList->setEditable( FALSE );
  curvesList->setMinimumWidth(140);

  fileToolbar->addSeparator();

  gridIcon = QPixmap(grid);
  QToolButton *fileGrid = new QToolButton(gridIcon, "Create new grid", 0, this,
                                          SLOT(slotNewGrid()), fileToolbar);
  
  fileToolbar->addSeparator();

  pointerIcon = QPixmap(pointer);
  selPointer = new QToolButton(pointerIcon, "Normal pointer", 0, this,
														   SLOT(slotSelPointer()), fileToolbar);

  axisPointIcon = QPixmap(axispoint);
  selAxisPoint = new QToolButton(axisPointIcon, "Operates on/adds an axis point", 0, this,
															   SLOT(slotSelAxisPoint()), fileToolbar);

  curvePointIcon = QPixmap(curvepoint);
  selCurvePoint = new QToolButton(curvePointIcon, "Operates on/adds a curve point", 0, this,
    															SLOT(slotSelCurvePoint()), fileToolbar);

	selPointer->setToggleButton( TRUE );
	selAxisPoint->setToggleButton( TRUE );
	selCurvePoint->setToggleButton( TRUE );
	
	selPointer->setOn( TRUE );
	selAxisPoint->setOn( FALSE );
	selCurvePoint->setOn( FALSE );

  fileToolbar->addSeparator();

  recycledIcon = QPixmap(recycled);
  QToolButton *viewRecycle = new QToolButton(recycledIcon, "Redraws everything", 0, this,
    											                   SLOT(slotRefresh()), fileToolbar);

  fileToolbar->addSeparator();

  QWhatsThis::whatsThisButton(fileToolbar);
  QWhatsThis::add(fileNew,"Click this button to create a new file.\n\n"
                  "You can also select the New command from the File menu.");
  QWhatsThis::add(fileOpen,"Click this button to open a new file.\n\n"
                  "You can also select the Open command from the File menu.");
  QWhatsThis::add(fileImport,"Click this button to import an image into the current file.");
  QWhatsThis::add(fileSave,"Click this button to save the file you are "
                  "editing. You will be prompted for a file name.\n\n"
                  "You can also select the Save command from the File menu.");

  QWhatsThis::add(fileExportSetup,"Click this button to change the way your data "
                  "will be exported.\n\n"
                  "You can also select the Export Setup command from the File menu.");
  QWhatsThis::add(fileExportPreview,"Click this button to pre-visualize how your exported data "
                  "will look like in the final export file.\n\n"
                  "You can also select the Export Preview command from the File menu.");
  QWhatsThis::add(fileExport,"Click this button to actually export your data to an ASCII file.\n\n"
                  "You can also select the Export command from the File menu.");

  QWhatsThis::add(fileAxis, "Click this button to create a new axes system.\n\n"
                  "Afterwards you will have to assign a total of three points "
                  "for this axes system to be operative.");
  QWhatsThis::add(axesList, "Select Axes system form the list of "
                  "axes systems defined in this document.");

  QWhatsThis::add(fileCurve, "Click this button to create a new curve.\n\n"
                  "Afterwards you will be able to assign points to this curve.");
  QWhatsThis::add(curvesList, "Select Curve form the list of curves "
                  "defined in this document.");

  QWhatsThis::add(fileGrid, "Click this button to create a new grid.");

  QWhatsThis::add(selPointer,"Click this button to select normal pointer.");
  QWhatsThis::add(selAxisPoint,"Click this button to change to 'Axis Point Mode'.\n\n"
  		                       "On this mode, the left mouse button add an axis point, "
  		                       "the middle button allows it to be moved, and the right "
  		                       "button displays the axis point parameters dialog.");
  QWhatsThis::add(selCurvePoint,"Click this button to change to 'Curve Point Mode'.\n\n"
  		                       "On this mode, the left mouse button add a curve point "
  		                       "(for the current selected curve), "
  		                       "the middle button allows it to be moved, and the right "
  		                       "button displays the curve point parameters dialog.");

  QWhatsThis::add(viewRecycle, "Click this button if the image gets dirty.");

  		
  connect(curvesList, SIGNAL(activated(int)), SLOT(slotSelCurve(int)));
  connect(axesList,   SIGNAL(activated(int)), SLOT(slotSelAxes(int)));
}

void QScanPlot::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //STATUSBAR
  statusBar()->message(IDS_STATUS_DEFAULT, 2000);
}

void QScanPlot::initDoc()
{
  doc=new QScanPlotDoc();
  exportFirstTime = true;
}

void QScanPlot::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // set the main widget here
  view=new QScanPlotView(this, doc);

  dialogExportSetup = new dExportSetup();

  setCentralWidget(view);

  view->setInitialZoom( normalized_zoom(oldz) );
  view->slotChangedPointerMode(ID_ON_POINTER);

  connect(this, SIGNAL(changedPointerMode(int)), view, SLOT(slotChangedPointerMode(int)));
  connect(view, SIGNAL(replaceAxes(QString, QString)), this, SLOT(slotReplaceAxes(QString, QString)));
  connect(view, SIGNAL(replaceCurve(QString, QString)), this, SLOT(slotReplaceCurve(QString, QString)));
  connect(doc,  SIGNAL(deletedCurve(int)), this, SLOT(slotRemoveCurveFromList(int)));
  connect(doc,  SIGNAL(deletedAxes(int)), this, SLOT(slotRemoveAxesFromList(int)));
}

bool QScanPlot::queryExit()
{
  int exit=QMessageBox::information(this, "Quit...",
                                    "Do your really want to quit?",
                                    QMessageBox::Ok, QMessageBox::Cancel);
  return (exit==1);
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void QScanPlot::slotFileNew()
{
  statusBar()->message("Creating new file...");
  doc->newDoc();
  exportFirstTime = true;
  enableCommands();
  updateToolbar();
  setCaption("<none>");
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void QScanPlot::slotFileOpen()
{
  statusBar()->message("Opening file...");

  QString fileName = QFileDialog::getOpenFileName(0,0,this);
  if (!fileName.isEmpty()) {
    doc->newDoc();
    doc->load(fileName);
    exportFirstTime = true;
    updateToolbar();
    enableCommands();
    setCaption(fileName);
    QString message="Loaded document: "+fileName;
    statusBar()->message(message, 2000);
  } else {
    statusBar()->message("Opening aborted", 2000);
  }
}

void QScanPlot::slotFileImportImage()
{
  statusBar()->message("Import image file...");

  QString fileName = QFileDialog::getOpenFileName(0,0,this);
  if (!fileName.isEmpty()) {
    doc->loadImage(fileName);
    view->slotDocumentChanged();
    QString message="Imported image: "+fileName;
    enableCommands();
    statusBar()->message(message, 2000);
  } else {
    statusBar()->message("Import aborted", 2000);
  }
}

void QScanPlot::slotFileSave()
{
  statusBar()->message("Saving file...");
  if ( doc->fileName() != "-" ) {
    doc->save();
  } else {
    slotFileSaveAs();
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void QScanPlot::slotFileSaveAs()
{
  statusBar()->message("Saving file under new filename...");
  QString fn = QFileDialog::getSaveFileName(0, 0, this);
  if (!fn.isEmpty()) {
    doc->saveAs(fn);
    setCaption(fn);
  } else {
    statusBar()->message("Saving aborted", 2000);
  }
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void QScanPlot::slotFileClose()
{
  statusBar()->message("Closing file...");
  slotFileSave();
  slotFileNew();
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void QScanPlot::slotFilePrint()
{
  statusBar()->message("Printing...");
  QPrinter printer;
  if (printer.setup(this)) {
    QPainter painter;
    painter.begin(&printer);

    ///////////////////////////////////////////////////////////////////
    // TODO: Define printing by using the QPainter methods here

    painter.end();
  };

  statusBar()->message(IDS_STATUS_DEFAULT);
}

void QScanPlot::slotFileQuit()
{ 
  statusBar()->message("Exiting application...");
  ///////////////////////////////////////////////////////////////////
  // exits the Application
  if(doc->isModified()) {
    if(queryExit()) {
      pref->setGroup("GUI");
      pref->setNumber("Window_Width", width());
      pref->setNumber("Window_Height", height());
      qApp->quit();
    } else {

    };
  } else {
    pref->setGroup("GUI");
    pref->setNumber("Window_Width", width());
    pref->setNumber("Window_Height", height());
    qApp->quit();
  };

  statusBar()->message(IDS_STATUS_DEFAULT);
}

void QScanPlot::slotEditCut()
{
  statusBar()->message("Cutting selection...");

  statusBar()->message(IDS_STATUS_DEFAULT);
}

void QScanPlot::slotEditCopy()
{
  statusBar()->message("Copying selection to clipboard...");
  
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void QScanPlot::slotEditPaste()
{
  statusBar()->message("Inserting clipboard contents...");
  
  statusBar()->message(IDS_STATUS_DEFAULT);
}


void QScanPlot::slotViewToolBar()
{
  statusBar()->message("Toggle toolbar...");
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  
  if (fileToolbar->isVisible()) {
    fileToolbar->hide();
    viewMenu->setItemChecked(ID_VIEW_TOOLBAR, false);
  } else {
    fileToolbar->show();
    viewMenu->setItemChecked(ID_VIEW_TOOLBAR, true);
  };

  statusBar()->message(IDS_STATUS_DEFAULT);
}

void QScanPlot::slotViewStatusBar()
{
  statusBar()->message("Toggle statusbar...");
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  
  if (statusBar()->isVisible()) {
    statusBar()->hide();
    viewMenu->setItemChecked(ID_VIEW_STATUSBAR, false);
  } else {
    statusBar()->show();
    viewMenu->setItemChecked(ID_VIEW_STATUSBAR, true);
  }
  
  statusBar()->message(IDS_STATUS_DEFAULT);
}

void QScanPlot::slotHelpAbout()
{
  /*QMessageBox::about(this,"About...", IDS_APP_ABOUT );*/
  dAbout dlg;
  QPixmap aPixmap(QSCANPLOT_IMAGE);
  dlg.setImage(aPixmap);
  dlg.exec();
}

void QScanPlot::slotChangeZoom(int z)
{
  viewMenu->setItemChecked(oldz, false);
  viewMenu->setItemChecked(z, true);
  view->setZoom( normalized_zoom(z) );
  oldz = z;
}

void QScanPlot::slotSetSort()
{
  sortPoints = !sortPoints;
  optMenu->setItemChecked(ID_OPT_SORT, sortPoints);
}

void QScanPlot::slotSetInterpolator()
{
  dInterpolator dlg;
  LagrangeInterpolator2nd l1st;
  LagrangeInterpolator2nd l2nd;
  LagrangeInterpolator3rd l3rd;
  dlg.addEntry("Lagrange 1st order",l1st.description());
  dlg.addEntry("Lagrange 2nd order",l2nd.description());
  dlg.addEntry("Lagrange 3rd order",l3rd.description());
  dlg.select(0);
  if (dlg.exec()) {
    nInterpolator = dlg.getInterpolator();
    //createInterpolator(nInterpolator);
  }
}

void QScanPlot::slotStatusHelpMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar temporary (text, msec)
  statusBar()->message(text, 2000);
}

void QScanPlot::statusCallback(int id_)
{
  switch (id_)
  {
    case ID_FILE_NEW:
         slotStatusHelpMsg("Creates a new document");
         break;

    case ID_FILE_OPEN:
         slotStatusHelpMsg("Opens an existing document");
         break;

    case ID_FILE_IMPORT:
         slotStatusHelpMsg("Imports an image to be used in the current document");
         break;

    case ID_FILE_SAVE:
         slotStatusHelpMsg("Saves the actual document");
         break;

    case ID_FILE_SAVE_AS:
         slotStatusHelpMsg("Saves the actual document as...");
         break;

    case ID_FILE_CLOSE:
         slotStatusHelpMsg("Closes the actual document");
         break;

    case ID_FILE_PRINT:
         slotStatusHelpMsg("Prints out the actual document");
         break;

    case ID_FILE_QUIT:
         slotStatusHelpMsg("Quits the application");
         break;

    case ID_EDIT_CUT:
         slotStatusHelpMsg("Cuts the selected section and puts it to the clipboard");
         break;

    case ID_EDIT_COPY:
         slotStatusHelpMsg("Copies the selected section to the clipboard");
         break;

    case ID_EDIT_PASTE:
         slotStatusHelpMsg("Pastes the clipboard contents to actual position");
         break;

    case ID_EDIT_SELECT_ALL:
         slotStatusHelpMsg("Selects the whole document contents");
         break;

    case ID_VIEW_TOOLBAR:
         slotStatusHelpMsg("Enables/disables the toolbar");
         break;

    case ID_VIEW_STATUSBAR:
         slotStatusHelpMsg("Enables/disables the statusbar");
         break;

    case ID_VIEW_ZOOM25:
    case ID_VIEW_ZOOM50:
    case ID_VIEW_ZOOM100:
    case ID_VIEW_ZOOM200:
    case ID_VIEW_ZOOM400:
         slotStatusHelpMsg("Changes zoom");
         break;

    case ID_ELEM_NEWAXES:
         slotStatusHelpMsg("Shows dialog for adding a new Axes system to the current document");
         break;

    case ID_ELEM_EDITAXES:
         slotStatusHelpMsg("Shows dialog to edit Axes systems properties");
         break;

    case ID_ELEM_NEWCURVE:
         slotStatusHelpMsg("Shows dialog for adding a new Curve to the current document");
         break;

    case ID_ELEM_EDITCURVES:
         slotStatusHelpMsg("Shows dialog to edit Curves properties");
         break;

    case ID_ELEM_NEWGRID:
         slotStatusHelpMsg("Shows dialog for adding a new Grid to the current document");
         break;

    case ID_ELEM_EDITGRIDS:
         slotStatusHelpMsg("Shows dialog to edit Grids properties");
         break;

    case ID_ELEM_EDITMODELS:
         slotStatusHelpMsg("Shows dialog to edit export models for curves");
         break;

    case ID_ON_POINTER:
         slotStatusHelpMsg("Changes to normal Pointer mode");
         break;

    case ID_ON_AXISPOINT:
         slotStatusHelpMsg("Changes to Axis Point mode");
         break;

    case ID_ON_CURVEPOINT:
         slotStatusHelpMsg("Changes to Curve Point mode");
         break;

    case ID_OPT_SORT:
         slotStatusHelpMsg("Activates sorting of curve points for export");
         break;

    case ID_OPT_INTERPOLATOR:
         slotStatusHelpMsg("Shows dialog for selecting active interpolator");
         break;

    case ID_OPT_PREFERENCES:
         slotStatusHelpMsg("Shows dialog to change user-defined preferences");
         break;

    case ID_TOOL_CREATEPLUGIN:
         slotStatusHelpMsg("Shows dialog to create plug-in functions at run time");
         break;

    case ID_HELP_ABOUT:
         slotStatusHelpMsg("Shows an aboutbox");
         break;
  }
}

/** Enable Pointer mode in Elements menu */
void QScanPlot::slotSelPointer()
{
  togglePointer(ID_ON_POINTER);
}

/** Enable Axis Point mode in Elements menu */
void QScanPlot::slotSelAxisPoint()
{
  togglePointer(ID_ON_AXISPOINT);
}

/** Enable Curve Point mode in Elements menu */
void QScanPlot::slotSelCurvePoint()
{
  togglePointer(ID_ON_CURVEPOINT);
}

/** Auxiliary function, to select one of the pointer modes */
void QScanPlot::togglePointer(int id)
{
	selPointer->setOn(id == ID_ON_POINTER);
	selAxisPoint->setOn(id == ID_ON_AXISPOINT);
	selCurvePoint->setOn(id == ID_ON_CURVEPOINT);
	emit changedPointerMode(id);
}

/** Updates toolbar presentation after opening new document */
void QScanPlot::updateToolbar()
{
  int i;

  curvesList->clear();
  for (i=0; i<doc->getNCurves(); i++) {
		curvesList->insertItem( doc->getCurve(i)->getName() );
  }
  curvesList->update();
  axesList->clear();
  for (i=0; i<doc->getNAxes(); i++) {
		axesList->insertItem( doc->getAxes(i)->getName() );
  }
  axesList->update();
}

/** Updates menus and toolbar */
void QScanPlot::enableCommands()
{
  bool existAxes   = doc->getNAxes() > 0;
  bool existCurves = doc->getNCurves() > 0;
  bool existGrids  = doc->getNGrids() > 0;
  bool existImage  = ! doc->getImage()->isNull();

  bool enableSave     = existAxes | existCurves | existGrids | existImage;
  bool enableExport   = existImage;
  bool enablePrint    = existImage;
  bool enableZoom     = existImage;
  bool enablePointers = existImage;

  fileMenu->setItemEnabled( ID_FILE_SAVE,            enableSave );
  fileMenu->setItemEnabled( ID_FILE_SAVE_AS,         enableSave );
  fileMenu->setItemEnabled( ID_FILE_CLOSE,           enableSave );
  fileMenu->setItemEnabled( ID_FILE_EXPORT,          enableExport );
  fileMenu->setItemEnabled( ID_FILE_EXPORTSETUP,     enableExport );
  fileMenu->setItemEnabled( ID_FILE_EXPORTPREVIEW,   enableExport );
  fileMenu->setItemEnabled( ID_FILE_PRINT,           enablePrint );

  viewMenu->setItemEnabled( ID_VIEW_ZOOM25,          enableZoom );
  viewMenu->setItemEnabled( ID_VIEW_ZOOM50,          enableZoom );
  viewMenu->setItemEnabled( ID_VIEW_ZOOM100,         enableZoom );
  viewMenu->setItemEnabled( ID_VIEW_ZOOM200,         enableZoom );
  viewMenu->setItemEnabled( ID_VIEW_ZOOM400,         enableZoom );

  elemMenu->setItemEnabled( ID_ON_AXISPOINT,         enablePointers );
  elemMenu->setItemEnabled( ID_ON_CURVEPOINT,        enablePointers );

  selAxisPoint->setEnabled( enablePointers );
  selCurvePoint->setEnabled( enablePointers );
}

/** Selects curve number c from combo box in toolbar */
void QScanPlot::slotSelCurve(int c)
{
  doc->selectCurve(c);
}

/** Selects curve number c from combo box in toolbar */
void QScanPlot::slotSelAxes(int a)
{
  doc->selectAxes(a);
}

/** Shows dialog for adding a new axes system to document */
void QScanPlot::slotNewAxes()
{
  dNewElement dlg;
  QColor col(Qt::blue);
  dlg.setTitle("New Axes System");
  dlg.setBtnColor(col);
  if ( dlg.exec() ) {
    Axes *a = new Axes();
    QString name(dlg.getName());
    QString desc(dlg.getDesc());
    col = dlg.getColor();
    a->setName( name );
    a->setDesc( desc );
    a->setColor( col );
    a->setScales(dlg.getXAxisScale(), dlg.getYAxisScale());
    doc->newAxes( a );
		axesList->insertItem( doc->getAxes(doc->getNAxes()-1)->getName() );
		int nNewAxes = axesList->count() - 1;
    axesList->setCurrentItem( nNewAxes );
    doc->selectAxes( nNewAxes );
    enableCommands();
  } else {
    statusBar()->message("Addition of new Axes system aborted", 2000);
  }
}

/** Shows dialog to edit axes properties */
void QScanPlot::slotEditAxes()
{
  dEditElement dlg;
  dlg.setMode(dEditElement::EditAxes);
  dlg.setDoc(doc);

  int n = doc->getNAxes();
  QString name, desc;
  QColor col;
  Axes *a;
  int i;
  for (i=0; i<n; i++) {
    a = doc->getAxes(i);
    name = a->getName();
    desc = a->getDesc();
    col  = a->getColor();
    dlg.addItem(name, desc, col, a->getXAxisScale(), a->getYAxisScale());
  }

  //launch editor
  if (dlg.exec()) {
		axesList->clear();
    for (i=0; i<n; i++) {
  		axesList->insertItem( doc->getAxes(i)->getName() );
  	}	
    view->slotDocumentChanged();
    doc->setModified(true);
  }
}

/** Deletes an axes */
void QScanPlot::slotRemoveAxesFromList(int i)
{
  axesList->removeItem( i );
  slotRefresh();
}

/** Shows dialog for adding a new curve to document */
void QScanPlot::slotNewCurve()
{
  dNewElement dlg;
  QColor col(Qt::red);
  dlg.setTitle("New Curve");
  dlg.setBtnColor(col);
  dlg.hideAxesScales();
  if ( dlg.exec() ) {
    Curve *c = new Curve();
    QString name(dlg.getName());
    QString desc(dlg.getDesc());
    col = dlg.getColor();
    c->setName( name );
    c->setDesc( desc );
    c->setColor( col );
    doc->newCurve( c );
		curvesList->insertItem( doc->getCurve(doc->getNCurves()-1)->getName() );
		int nNewCurve = curvesList->count() - 1;
		curvesList->setCurrentItem( nNewCurve );
    doc->selectCurve( nNewCurve );
    enableCommands();
  } else {
    statusBar()->message("Addition of new Curve aborted", 2000);
  }	
}

/** Deletes a curve */
void QScanPlot::slotRemoveCurveFromList(int i)
{
  curvesList->removeItem( i );
  slotRefresh();
}

/** Shows dialog to edit curves properties */
void QScanPlot::slotEditCurves()
{
  dEditElement dlg;
  dlg.setMode(dEditElement::EditCurve);
  dlg.setDoc(doc);

  int n = doc->getNCurves();
  QString name, desc;
  QColor col;
  Curve *c;
  int i;
  for (i=0; i<n; i++) {
    c = doc->getCurve(i);
    name = c->getName();
    desc = c->getDesc();
    col  = c->getColor();
    dlg.addItem(name, desc, col, 0, 0);
  }

  //launch editor
  if (dlg.exec()) {
		curvesList->clear();
    for (i=0; i<n; i++) {
  		curvesList->insertItem( doc->getCurve(i)->getName() );
  	}	
    view->slotDocumentChanged();
    doc->setModified(true);
  }
}

/** Shows dialog for adding a new grid to document */
void QScanPlot::slotNewGrid()
{
  dGrid dlg;
  if ( dlg.exec() ) {
    Grid *g = new Grid();
    QString name(dlg.getName());
    QString desc(dlg.getDesc());
    g->setName( name );
    g->setDesc( desc );
    g->init( dlg.getXmin(), dlg.getXmax(), dlg.getNIntervals() );
    doc->newGrid( g );
    enableCommands();
  } else {
    statusBar()->message("Addition of new Grid aborted", 2000);
  }	
}

/** Shows dialog to edit grid properties */
void QScanPlot::slotEditGrids()
{
  dEditElement dlg;
  dlg.setMode(dEditElement::EditGrid);
  dlg.setDoc(doc);

  int n = doc->getNGrids();
  QString name, desc;
  Grid *g;
  int i;
  for (i=0; i<n; i++) {
    g = doc->getGrid(i);
    name = g->getName();
    desc = g->getDesc();
    dlg.addItem(name, desc, Qt::black, 0, 0,
                g->getXmin(), g->getXmax(), g->getNIntervals(), g->getIntervalWidth());
  }

  //launch editor
  if (dlg.exec()) {
    view->slotDocumentChanged();
    doc->setModified(true);
  }
}

/** Shows dialog to edit export models for curves */
void QScanPlot::slotEditModels()
{
  dModels dlg;

  dlg.setFunctions(Plugins_List);
  dlg.createModels(doc->getNCurves());

  Curve *c;
  Model *m;
  int i;
  for (i=0; i<doc->getNCurves(); i++) {
    c = doc->getCurve(i);
    m = const_cast<Model*>(&(c->getModel()));
    dlg.setCurveModel(i, c->getName(), m->getModel(), m->getFx(), m->getFy());
  }

  //launch editor
  if ( dlg.exec() ) {
    for (i=0; i<doc->getNCurves(); i++) {
      c = doc->getCurve(i);
      m = const_cast<Model*>(&(c->getModel()));
      m->setModel( dlg.getCurveModel(i) );
      m->setFx( dlg.getCurveModelFx(i) );
      m->setFy( dlg.getCurveModelFy(i) );
    }
  }
}

void QScanPlot::slotReplaceAxes(QString s1, QString s2)
{
  for (int i=0; i<axesList->count(); i++) {
    if (axesList->text(i) == s1) {
      axesList->changeItem(s2, i);
      break;
    }
  }
}

void QScanPlot::slotReplaceCurve(QString s1, QString s2)
{
  for (int i=0; i<curvesList->count(); i++) {
    if (curvesList->text(i) == s1) {
      curvesList->changeItem(s2, i);
      break;
    }
  }
}

/** Calls view->slotRefresh to redraw everything */
void QScanPlot::slotRefresh()
{
  view->slotRefresh();
}

/** No descriptions */
void QScanPlot::createInterpolator(int type, double*x, double*y, int n)
{
  if (interpolator != 0)
    delete interpolator;

  switch (type) {
  case INTERPOLATOR_LAGRANGE1ST:
    interpolator = new LagrangeInterpolator1st(x, y, n);
    break;
  case INTERPOLATOR_LAGRANGE2ND:
    interpolator = new LagrangeInterpolator2nd(x, y, n);
    break;
  case INTERPOLATOR_LAGRANGE3RD:
    interpolator = new LagrangeInterpolator3rd(x, y, n);
    break;
  default:
    interpolator = new LagrangeInterpolator3rd(x, y, n);
    break;
  }
}

/** Shows dialog of setup for exporting */
void QScanPlot::slotFileExportSetup()
{
  dExportSetup& dlg = *dialogExportSetup;
  int i;

  includedCurves.clear();
  excludedCurves.clear();

  if (exportFirstTime) {
    exportFirstTime = false;
    for (i=0; i<doc->getNCurves(); i++)
      includedCurves.append(i);
    axesSystem2Use         = 0;
    grid2Use               = 0;
    exportPointsCriteria   = dExportSetup::X1Y1X2Y2;
    exportLayout           = dExportSetup::AllInOneLine;
    exportSeparator        = dExportSetup::Space;
    exportComment          = dExportSetup::Number;
    exportNumeric          = dExportSetup::Automatic;
    exportSpecialFormat    = dExportSetup::NoSpecial;
  }

  ExportList::Iterator it;
  for( it = includedCurves.begin(); it != includedCurves.end(); ++it )
    dlg.includeItem( doc->getCurve(*it)->getName() );
  for( it = excludedCurves.begin(); it != excludedCurves.end(); ++it )
    dlg.excludeItem( doc->getCurve(*it)->getName() );

  dlg.clearAxesSystem();
  for (i=0; i<doc->getNAxes(); i++)
    dlg.addAxesSystem(doc->getAxes(i)->getName());

  dlg.clearGrid();
  for (i=0; i<doc->getNGrids(); i++)
    dlg.addGrid(doc->getGrid(i)->getName());

  dlg.init(axesSystem2Use, grid2Use, exportPointsCriteria, exportLayout,
           exportSeparator, exportNumeric, exportComment, exportSpecialFormat);


  if ( ! dlg.exec() )
    return;

  includedCurves.clear();
  excludedCurves.clear();

  int j;
  QString name;

  for (j=0; j<dlg.getNIncluded(); j++) {
    name = dlg.getIncluded(j);
    for (i=0; i<doc->getNCurves(); i++) {
      if (doc->getCurve(i)->getName() == name) {
        includedCurves.append(i);
        break;
      }
    }
  }

  for (j=0; j<dlg.getNExcluded(); j++) {
    name = dlg.getExcluded(j);
    for (i=0; i<doc->getNCurves(); i++) {
      if (doc->getCurve(i)->getName() == name) {
        excludedCurves.append(i);
        break;
      }
    }
  }

  axesSystem2Use         = dlg.getAxesSystem();
  grid2Use               = dlg.getGrid();
  exportPointsCriteria   = dlg.getCriteria();
  exportLayout           = dlg.getLayout();
  exportSeparator        = dlg.getSeparator();
  exportComment          = dlg.getComment();
  exportNumeric          = dlg.getNumeric();
  exportSpecialFormat    = dlg.getSpecialFormat();

  exportSeparatorString  = dlg.getSeparatorString();
  exportCommentString    = dlg.getCommentString();
  exportUserFormatString = dlg.getUserFormatString();

}

/** Export preview */
void QScanPlot::slotPreview()
{
  if (exportCommentString.isEmpty())
    return;

  QString strpreview;
  QTextStream stream( &strpreview, IO_WriteOnly );
  stream << flush;

  // Call the actual exporting routine
  exportDoIt(stream);

  stream << flush;

  dExportPreview dlg(this);
  dlg.setPreviewText(strpreview);
  dlg.exec();
}

/** Export data */
void QScanPlot::slotFileExport()
{
  QString fn = QFileDialog::getSaveFileName(0, 0, this);
  if (fn.isEmpty()) {
    statusBar()->message("Saving aborted", 2000);
    return;
  }

  QFile f( fn );
  if (!f.open( IO_WriteOnly )) {
    statusBar()->message("Cannot open file", 2000);
    return;
  }
  QTextStream stream( &f );

  // Call the actual exporting routine
  exportDoIt(stream);

  stream << flush;
  f.close();
}

/** The actual exporting routine */
void QScanPlot::exportDoIt(QTextStream &stream)
{
  CurvePoint *p;

  int numcurves = includedCurves.count();

  int maxnpairs = -1;
  ExportList::Iterator it;
  for( it = includedCurves.begin(); it != includedCurves.end(); ++it ) {
    doc->projectData(axesSystem2Use, *it);
    //if (sortPoints)
    //  doc->getCurve(*it)->getPointList()->sort();
    maxnpairs = max(maxnpairs, doc->getCurve(*it)->getNPoints());
  }

  Grid * g2use = doc->getGrid(grid2Use);
  Grid::GridValue x0;

  // create matrices mx and my with data point physical coordinates
  double *mx[numcurves];
  double *my[numcurves];
  int *npairs;

  QString curvenames[numcurves];
  PointList *pList;
  int ki, kj;
  int lastki = numcurves-1;
  int lastkj;

  double *zx[numcurves];
  double *zy[numcurves];
  int    *zn;
  Model  *zmodels[numcurves];

  for (ki=0; ki<numcurves; ki++) {
    mx[ki] = new double[maxnpairs];
    my[ki] = new double[maxnpairs];
  }

  npairs = new int[numcurves];

  // traverse all curves selected to be exported
  // scale then, and interpolate when needed

  for( it = includedCurves.begin(), ki = 0;
       it != includedCurves.end();
       ++it, ki++ ) {

    // get name
    curvenames[ki] = doc->getCurve(*it)->getName();

    // get models
    zmodels[ki] = new Model(doc->getCurve(*it)->getModel());

    // sort points if selected
    if ( sortPoints )
      pList = doc->getCurve(*it)->getPointSortedList();
    else
      pList = doc->getCurve(*it)->getPointList();

    // fill matrices with values from current curve
    PointListIterator pit(*pList);
    for ( kj = 0; pit.current(); ++pit, kj++ ) {
      p = pit.current();
      mx[ki][kj] = p->getX();
      my[ki][kj] = p->getY();
      doScale(&(mx[ki][kj]),
              &(my[ki][kj]),
              doc->getAxes(axesSystem2Use)->getXAxisScale(),
              doc->getAxes(axesSystem2Use)->getYAxisScale());
    }

    // go back to user-entered ordenation
    if (sortPoints)
      doc->getCurve(*it)->resetPointSortedList();

    // save number of data points for current curves (in order to speed up things)
    npairs[ki] = kj;

    // create output matrices
    switch ( exportPointsCriteria ) {

    case dExportSetup::X1Y1X2Y2:
    {
      if (ki == 0)
        zn = npairs;

      zx[ki] = mx[ki];
      zy[ki] = my[ki];

      break;
    }
    case dExportSetup::XY1Y2:
    {
      if (ki == 0) {

        zn = new int[numcurves];
        zx[0] = mx[0];
        zy[0] = my[0];
        zn[0] = npairs[0];

      } else {

        // create vectors for current curve
        zn[ki] = zn[0];
        zx[ki] = new double[ zn[ki] ];
        zy[ki] = new double[ zn[ki] ];

        // create interpolator
        createInterpolator(nInterpolator, mx[ki], my[ki], npairs[ki]);

        // interpolate with X values from first curve
        for ( kj = 0; kj < zn[0]; kj++) {
          zx[ki][kj] = zx[0][kj];
          zy[ki][kj] = interpolator->interpolate(zx[0][kj]);
        }
      }
      break;
    }
    case dExportSetup::GXY1Y2:
    {
      if (ki == 0) {
        zn = new int[numcurves];
        maxnpairs = g2use->getNPoints();
      }

      zn[ki] = maxnpairs;
      zx[ki] = new double[ zn[ki] ];
      zy[ki] = new double[ zn[ki] ];

      // create interpolator
      createInterpolator(nInterpolator, mx[ki], my[ki], npairs[ki]);

      // interpolate with X values from grid
      g2use->reset();
      for ( x0=g2use->begin(), kj=0; g2use->continues(); x0=g2use->next(), kj++ ) {
        zx[ki][kj] = x0;
        zy[ki][kj] = interpolator->interpolate(x0);
      }
      break;
    }
    default:
      break;

    } // end switch

  } // end for each selected curve

  // De-scale
  for (ki=0; ki<numcurves; ki++)
    for (kj=0; kj<zn[ki]; kj++)
      undoScale(&(zx[ki][kj]), &(zy[ki][kj]),
                doc->getAxes(axesSystem2Use)->getXAxisScale(),
                doc->getAxes(axesSystem2Use)->getYAxisScale());

  // Apply user functions
  QString plgName;
  Plugin *plgFx=0, *plgFy=0;
  Model *m;

  for (ki=0; ki<numcurves; ki++) {

    m = zmodels[ki];

    if ((m->getModel() & MODEL_FX_Y) > 0) {
      plgFx = Plugins_Factory[ Plugins_List[m->getFx()] ]();
      for (kj=0; kj<zn[ki]; kj++)
        if (plgFx > 0) { zx[ki][kj] = plgFx->f( zx[ki][kj] ); }
      delete plgFx;
      plgFx = 0;

    if ((m->getModel() & MODEL_X_GY) > 0)
      plgFy = Plugins_Factory[ Plugins_List[m->getFy()] ]();
      for (kj=0; kj<zn[ki]; kj++)
        if (plgFy > 0) { zy[ki][kj] = plgFy->f( zy[ki][kj] ); }
      delete plgFy;
      plgFy = 0;
    }

  }

  // Get number format for output and NaN identifier
  QString number;
  char numFormat[21];
  if ( exportNumeric != 0 ) {
    strncpy(numFormat, exportUserFormatString.latin1(), 20);
  } else {
    strncpy(numFormat, "%g", 20);
  }
  number.sprintf(numFormat, -1.1); /* force to have at leas 5 chars for NaN string */
  QString NaN(number.sprintf("%*s", number.length(), "NaN"));

  //--------------------------------------------------------------------
  // Here the export starts !!

  //////////////////////////////////////////////////////////////////////
  // OCTAVE ASCII FORMAT

  if ( exportSpecialFormat == dExportSetup::Octave ) {

    stream << "# Created by QScanPlot " VERSION "\n";

    switch ( exportLayout ) {

    case dExportSetup::OnePerLine:

      for( ki=0; ki<numcurves; ki++) {

        stream << "# name: " << curvenames[ki] << '\n';
        stream << "# type: matrix\n";
        stream << "# rows: " << zn[ki] << '\n';
        stream << "# columns: 2\n";

        for (kj=0; kj<zn[ki]; kj++) {
          //stream << zx[ki][kj] << ' ' << zy[ki][kj] << '\n';
          stream << number.sprintf(numFormat,zx[ki][kj]) << ' ';
          stream << number.sprintf(numFormat,zy[ki][kj]) << '\n';
        }

      }

      break;

    case dExportSetup::AllInOneLine:

      stream << "# name: data\n";
      stream << "# type: matrix\n";
      stream << "# rows: " << maxnpairs << '\n';
      stream << "# columns: "
             << ((exportPointsCriteria == dExportSetup::X1Y1X2Y2) ? 2*numcurves : numcurves+1)
             << '\n';

      for (kj=0; kj<maxnpairs; kj++) {
        for (ki=0; ki<numcurves; ki++) {
          if (kj < zn[ki]) {
            if ( ki == 0 )
              stream << number.sprintf(numFormat,zx[ki][kj]) << ' ';
            else if (exportPointsCriteria == dExportSetup::X1Y1X2Y2)
              stream << number.sprintf(numFormat,zx[ki][kj]) << ' ';
            stream << number.sprintf(numFormat,zy[ki][kj]) << ((ki != lastki) ? ' ' : '\n');
          } else {
            if ( ki == 0 )
              stream << NaN << ' ';
            else if (exportPointsCriteria == dExportSetup::X1Y1X2Y2)
              stream << NaN << ' ';
            stream << NaN << ((ki != lastki) ? ' ' : '\n');
          }
        }
      }

      break;

    default:
      break;
    }

    return;

  }

  //////////////////////////////////////////////////////////////////////
  // SCILab SCRIPT FORMAT

  if (exportSpecialFormat == dExportSetup::SCILab) {

    stream << "// Created by QScanPlot " VERSION "\n";
    stream << "// curves: " << numcurves << '\n';

    switch ( exportLayout ) {

    case dExportSetup::OnePerLine:

      for( ki=0; ki<numcurves; ki++) {

        stream << '\n';
        stream << "// name: " << curvenames[ki] << '\n';
        stream << "// rows: " << zn[ki] << '\n';

        stream << curvenames[ki] << " = [\n";

        lastkj = zn[ki] - 1;

        for (kj=0; kj<zn[ki]; kj++) {
          stream << ' '  << number.sprintf(numFormat,zx[ki][kj]);
          stream << ", " << number.sprintf(numFormat,zy[ki][kj]) << ((kj != lastkj) ? ";\n" : "];\n");
        }
      }

      break;

    case dExportSetup::AllInOneLine:

      stream << "// format: "
             << ((exportPointsCriteria == dExportSetup::X1Y1X2Y2) ? "X1Y1X2Y2" :
                 ((exportPointsCriteria == dExportSetup::XY1Y2) ? "XY1Y2" : "GXY1Y2"))
             << '\n';
      stream << "// names: ";

      for( ki=0; ki<numcurves; ki++ ) {
        if ( ki == 0 )
          stream << "x" << curvenames[ki] << ", ";
        else if (exportPointsCriteria == dExportSetup::X1Y1X2Y2)
          stream << "x" << curvenames[ki] << ", ";
        stream << "y" << curvenames[ki] << ((ki != lastki) ? ", " : "\n");
      }

      stream << "data = [\n";

      for (kj=0; kj<maxnpairs; kj++) {
        for (ki=0; ki<numcurves; ki++) {

          lastkj = zn[ki] - 1;

          if (kj < zn[ki]) {
            if ( ki == 0 )
              stream << number.sprintf(numFormat,zx[ki][kj]) << ", ";
            else if (exportPointsCriteria == dExportSetup::X1Y1X2Y2)
              stream << number.sprintf(numFormat,zx[ki][kj]) << ", ";
            stream << number.sprintf(numFormat,zy[ki][kj]);
          } else {
            if ( ki == 0 )
              stream << NaN << ", ";
            else if (exportPointsCriteria == dExportSetup::X1Y1X2Y2)
              stream << NaN << ", ";
            stream << NaN;
          }

          stream << ((ki != lastki) ? ", " :
                     ((kj != lastkj) ? ";\n" : "];\n"));
        }
      }

      break;

    default:
      break;
    }

    return;

  }

  //////////////////////////////////////////////////////////////////////
  // GNUPlot FORMAT

  if (exportSpecialFormat == dExportSetup::GNUPlot) {

    stream << "# Created by QScanPlot " VERSION "\n";
    stream << "# curves: " << numcurves << '\n';

    switch ( exportLayout ) {

    case dExportSetup::OnePerLine:

      for( ki=0; ki<numcurves; ki++) {

        stream << '\n';
        stream << "# name: " << curvenames[ki] << '\n';
        stream << "# rows: " << zn[ki] << '\n';

        lastkj = zn[ki] - 1;

        for (kj=0; kj<zn[ki]; kj++) {
          stream << ' '  << number.sprintf(numFormat,zx[ki][kj]);
          stream << ", " << number.sprintf(numFormat,zy[ki][kj]) << '\n';
        }
        stream << '\n';

      }

      break;

    case dExportSetup::AllInOneLine:

      stream << "# format: "
             << ((exportPointsCriteria == dExportSetup::X1Y1X2Y2) ? "X1Y1X2Y2" :
                 ((exportPointsCriteria == dExportSetup::XY1Y2) ? "XY1Y2" : "GXY1Y2"))
             << '\n';
      stream << "# names: ";

      for( ki=0; ki<numcurves; ki++ ) {
        if ( ki == 0 )
          stream << "x" << curvenames[ki] << ", ";
        else if (exportPointsCriteria == dExportSetup::X1Y1X2Y2)
          stream << "x" << curvenames[ki] << ", ";
        stream << "y" << curvenames[ki] << ((ki != lastki) ? ", " : "\n");
      }

      stream << "# rows: " << maxnpairs << '\n';
      stream << '\n';

      for (kj=0; kj<maxnpairs; kj++) {
        for (ki=0; ki<numcurves; ki++) {

          lastkj = zn[ki] - 1;

          if (kj < zn[ki]) {
            if ( ki == 0 )
              stream << number.sprintf(numFormat,zx[ki][kj]) << ", ";
            else if (exportPointsCriteria == dExportSetup::X1Y1X2Y2)
              stream << number.sprintf(numFormat,zx[ki][kj]) << ", ";
            stream << number.sprintf(numFormat,zy[ki][kj]) << ((ki != lastki) ? ", " : "\n");
          } else {
            if ( ki == 0 )
              stream << NaN << ", ";
            else if (exportPointsCriteria == dExportSetup::X1Y1X2Y2)
              stream << NaN << ", ";
            stream << NaN << ((ki != lastki) ? ", " : "\n");
          }
        }
      }

      break;

    default:
      break;
    }

    return;

  }

  //////////////////////////////////////////////////////////////////////
  // USER DEFINED FORMAT

  stream << exportCommentString << "Created by QScanPlot " VERSION "\n";

  switch ( exportLayout ) {

  case dExportSetup::OnePerLine:

    for( ki=0; ki<numcurves; ki++) {

      stream << exportCommentString   << "x" << curvenames[ki];
      stream << exportSeparatorString << "y" << curvenames[ki] << '\n';

      for (kj=0; kj<zn[ki]; kj++) {
        stream << ' ' << number.sprintf(numFormat,zx[ki][kj]);
        stream << exportSeparatorString << number.sprintf(numFormat,zy[ki][kj]) << '\n';
      }

    }

    break;

  case dExportSetup::AllInOneLine:

    stream << exportCommentString << "format: "
           << ((exportPointsCriteria == dExportSetup::X1Y1X2Y2) ? "X1Y1X2Y2" :
               ((exportPointsCriteria == dExportSetup::XY1Y2) ? "XY1Y2" : "GXY1Y2"))
           << '\n';
    stream << exportCommentString << "names: ";

    for( ki=0; ki<numcurves; ki++ ) {
      if ( ki == 0 )
        stream << "x" << curvenames[ki] << exportSeparatorString;
      else if (exportPointsCriteria == dExportSetup::X1Y1X2Y2)
        stream << "x" << curvenames[ki] << exportSeparatorString;
      stream << "y" << curvenames[ki] << ((ki != lastki) ? exportSeparatorString : QString("\n"));
    }

    stream << exportCommentString << "columns: "
           << ((exportPointsCriteria == dExportSetup::X1Y1X2Y2) ? 2*numcurves : numcurves+1)
           << '\n';
    stream << exportCommentString << '\n';

    for (kj=0; kj<maxnpairs; kj++) {
      for (ki=0; ki<numcurves; ki++) {
        if (kj < zn[ki]) {
          if ( ki == 0 )
            stream << number.sprintf(numFormat,zx[ki][kj]) << exportSeparatorString;
          else if (exportPointsCriteria == dExportSetup::X1Y1X2Y2)
            stream << number.sprintf(numFormat,zx[ki][kj]) << exportSeparatorString;
          stream << number.sprintf(numFormat,zy[ki][kj]) << ((ki != lastki) ? exportSeparatorString : QString("\n"));
        } else {
          if ( ki == 0 )
            stream << NaN << exportSeparatorString;
          else if (exportPointsCriteria == dExportSetup::X1Y1X2Y2)
            stream << NaN << exportSeparatorString;
          stream << NaN << ((ki != lastki) ? exportSeparatorString : QString("\n"));
        }
      }
    }

    break;

  default:
    break;

  } // Layout

  return;
}

/** No descriptions */
void QScanPlot::slotAboutQt()
{
  QMessageBox::aboutQt( this, "Trollies' Qt" );
}

/** This function loads all the plugins defined by the user and
 *  stored in the specified directory */
bool QScanPlot::loadPlugins()
{
  bool bStatus = true;

  pref->setGroup("GUI");
  QDir d(pref->getString("Plugins_Directory"), "*.so");

  if ( !d.exists() ) {

    qWarning( "Cannot find the plugins directory" );
    bStatus = false;

  } else {

    numPlugins = 0;
    Plugins_List.clear();
    Plugins_Factory.clear();

    Plugins_List[0] = QString("none");
    numPlugins++;

    d.setFilter( QDir::Files | QDir::NoSymLinks );
    d.setSorting( QDir::Time );

    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );      // create list iterator
    QFileInfo *fi;                          // pointer for traversing

    while ( (fi=it.current()) ) {           // for each .so file...

      hPlugins_Handle = dlopen(fi->absFilePath().data(), RTLD_NOW);

      if ( ! hPlugins_Handle ) {

        cerr << "Error loading " << fi->fileName().data() << ":\t"
             << dlerror() << endl << flush;

      } else {

        QString plgName = *static_cast<QString*>(dlsym(hPlugins_Handle, "Plugin_Name"));
        cerr << fi->fileName().data() << " loaded (implementing "
             << plgName.data() << ") . . ." << endl;
        Plugins_List[numPlugins] = plgName;
        numPlugins++;

      }
      ++it; // goto next list element
    }

  }

  return bStatus;
}

/** Read property of Preferences * pref */
Preferences * QScanPlot::get_preferences() {
	return pref;
}

/** Write property of Preferences * pref */
void QScanPlot::set_preferences( Preferences * _newVal)
{
  pref = _newVal;

  if (! pref->fileState()) {
    pref->setGroup("GUI");
    pref->setString("Plugins_Directory", "/tmp");
    pref->setBool("Splash", true);
    pref->setNumber("GUI_Style", 5);
    pref->setNumber("Window_Width", 650);
    pref->setNumber("Window_Height", 450);
    pref->setNumber("Initial_Zoom", ID_VIEW_ZOOM50 - ID_NORMALIZE_ZOOM);
    pref->setString("Background_Color", "lightgray");

    QFont f("helvetica", 10);
    pref->setString("Font_Family", f.family());
    pref->setNumber("Font_Size", f.pointSize());
    pref->setNumber("Font_Weight", f.weight());
    pref->setBool("Font_Italic", f.italic());
    pref->flush();
  }

  apply_preferences();
  resize(pref->getNumber("Window_Width"), pref->getNumber("Window_Height"));
  oldz = pref->getNumber("Initial_Zoom") + ID_NORMALIZE_ZOOM;
  viewMenu->setItemChecked(oldz, true);
  view->setInitialZoom( normalized_zoom(oldz) );
}

/** Apply changes done in preferences */
void QScanPlot::apply_preferences(void)
{
  pref->setGroup("GUI");

  switch ( pref->getNumber("GUI_Style") ) {
  case 0:
    qApp->setStyle( new QMotifStyle );
    break;

  case 1:
    qApp->setStyle( new QMotifPlusStyle );
    break;

  case 2:
    qApp->setStyle( new QWindowsStyle );
    break;

  case 3:
    qApp->setStyle( new QSGIStyle );
    break;

  case 4:
    qApp->setStyle( new QCDEStyle );
    break;

  case 5:
  default:
    qApp->setStyle( new QPlatinumStyle );
    break;

  }

  QColor col(pref->getString("Background_Color"));
  QPalette pal(col, col);

  qApp->setPalette(pal, true);

  QFont f;
  f.setFamily( pref->getString("Font_Family") );
  f.setPointSize( pref->getNumber("Font_Size") );
  f.setWeight( pref->getNumber("Font_Weight") );
  f.setItalic( pref->getBool("Font_Italic") );

  qApp->setFont( f, true );
}

/** Shows dialog to change user preferences */
void QScanPlot::slotPreferences()
{
  int id_zooms[] = { ID_VIEW_ZOOM25,
                     ID_VIEW_ZOOM50,
                     ID_VIEW_ZOOM100,
                     ID_VIEW_ZOOM200,
                     ID_VIEW_ZOOM400 };

  dPreferences dlg;

  pref->setGroup("GUI");
  dlg.set_pluginsdir( pref->getString("Plugins_Directory") );
  dlg.set_splash( pref->getBool("Splash") );
  dlg.set_GUI_style( pref->getNumber("GUI_Style") );
  dlg.set_color( pref->getString("Background_Color") );
  int initz = pref->getNumber("Initial_Zoom") + ID_NORMALIZE_ZOOM;
  for(int i=0; i<5; i++) {
    if (id_zooms[i] == initz) dlg.set_zoom(i);
  }

  //launch preferences dialog
  if ( dlg.exec() ) {
    pref->setString("Plugins_Directory", dlg.get_pluginsdir());
    pref->setBool("Splash", dlg.get_splash());
    pref->setNumber("GUI_Style", dlg.get_GUI_style());
    pref->setNumber("Initial_Zoom", id_zooms[dlg.get_zoom()] - ID_NORMALIZE_ZOOM);
    pref->setString("Background_Color", dlg.get_color());
    QFont f( dlg.get_font() );
    pref->setString("Font_Family", f.family());
    pref->setNumber("Font_Size", f.pointSize());
    pref->setNumber("Font_Weight", f.weight());
    pref->setBool("Font_Italic", f.italic());
    pref->flush();

    apply_preferences();
  }
}

/** Shows dialog to create new plugin */
void QScanPlot::slotCreatePlugin()
{
  bool at_least_one = false;
  bool yet_another = false;
  bool new_dialog = true;
  dNewPlugin *dlg = 0;

  do {

    if (new_dialog) { dlg = new dNewPlugin; }
    yet_another = dlg->exec();

    if (yet_another) {

      if ( dlg->get_name().isEmpty() ) {

        QMessageBox::information( this, "Enter a name",
                                  "You must provide a name for your plugin." );
        new_dialog = false;

      } else {

        QString plgName( dlg->get_name() );
        QString filename( pref->getString("Plugins_Directory") + plgName.lower() + ".cpp" );
        QFile f( filename );

        if ( ! f.open( IO_WriteOnly ) ) {
          QMessageBox::warning( this, "Error", "Couldn't open file " + filename);
          new_dialog = false;
          continue;
        }

        QTextStream t(&f);

        t << "// This is -*- C++ -*- " << endl;
        t << "// Source file for QScanPlot plug-in \"" << plgName << "\"" << endl;
        t << "// " << QDateTime::currentDateTime().toString() << endl;
        t << "" << endl;
        t << "#include \"plugin.h\"" << endl;
        t << "" << endl;
        t << "DECLARE_PLUGIN(" << plgName << ", \"" << plgName << "\")" << endl;
        t << "" << endl;
        t << "class " << plgName << " : public Plugin { " << endl;
        t << "public:" << endl;
        t << "  " << plgName << "() {}" << endl;
        t << "  ~" << plgName << "() {}" << endl;
        t << "  " << endl;
        t << "public:" << endl;
        t << "  double f(double x) {" << endl;
        t << dlg->get_definition() << endl;
        t << "  }" << endl;
        t << "};" << endl;
        t << "" << endl;
        t << "GENERATE_BUILDER(" << plgName << ")" << endl;

        f.close();

        QMessageBox::information( this, "Plugin source creation succesful",
                                  "The source file for your plug-in \"" + plgName + "\"\n has been created.\n\n"
                                  "You can find it at \n" + filename);

        at_least_one = generate_shared_library(plgName.latin1());

        if (! at_least_one) {
          QMessageBox::warning( this, "Error",
                                "Couldn't create the shared library\n"
                                "at directory " + pref->getString("Plugins_Directory") +
                                "\nPlease, check your installation and/or your settings.");
          yet_another = false;
        }

        new_dialog = true;
        delete dlg;
      }

    }

  } while (yet_another);

  delete dlg;
  dlg = 0;

  if (at_least_one) {
    QMessageBox::information( this, "New plugins are created",
                             "Your plugins directory will be scanned in order to\n"
                             "load the new plugins that were created.\n\n"
                             "Note that this could lead to some incoherences between\n"
                             "your new list of plugins and your selected plugins to\n"
                             "be used with an opened document.");
    loadPlugins();
  }
}

bool QScanPlot::generate_shared_library(const char *name)
{
  char *batch = "/tmp/batch-create-plugin.sh";
  char *batcherr = "/tmp/batch-create-plugin.err";

  QString filename(batch);
  QFile f( filename );

  if ( ! f.open( IO_WriteOnly ) ) {
    return false;
  }

  QTextStream t(&f);
  t << "cd " << pref->getString("Plugins_Directory") << endl;
  t << "make-plugin.sh " << name << " 1>" << batcherr << " 2>&1" << endl;
  f.close();

  if ( chmod(batch, 00744) == 0) {

    system(batch);

  } else {
    qWarning("Couldn't give execution permission to batch file " + QString(batch) +
             "\nRun 'make-plugin.sh " + QString(name) + "' in your plugins directory.");
    perror("generate_shared_library");
    return false;
  }

  return true;
}
