/*************************************************************************
 *                                                                          
 * qscanplot.h  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#ifndef QSCANPLOT_H
#define QSCANPLOT_H

// include files for QT
#include <qstring.h>
#include <qpopupmenu.h>
#include <qmainwindow.h>
#include <qaccel.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qpixmap.h>
#include <qtoolbutton.h>
#include <qapp.h>
#include <qstatusbar.h>
#include <qwhatsthis.h>
#include <qmsgbox.h>
#include <qfiledialog.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qcombobox.h>

// application specific includes
#include "qscanplotview.h"
#include "qscanplotdoc.h"
#include "resource.h"
#include "interpolator.h"
#include "lagrangeinterpolation.h"
#include "plugin.h"
#include "preferences.h"

class dExportSetup;

/**
  * This Class is the base class for your application. It sets up the main
  * window and providing a menubar, toolbar
  * and statusbar. For the main view, an instance of class QScanPlotView is
  * created which creates your view.
  */
class QScanPlot : public QMainWindow
{
  Q_OBJECT
  
public:
  /** construtor */
  QScanPlot();
  /** destructor */
  ~QScanPlot();
  /** initMenuBar creates the menu_bar and inserts the menuitems */
  void initMenuBar();
  /** this creates the toolbars. Change the toobar look and add new toolbars in this
   * function */
  void initToolBar();
  /** setup the statusbar */
  void initStatusBar();
  /** setup the document*/
  void initDoc();
  /** setup the mainview*/
  void initView();

  /** overloaded for Message box on last window exit */
  bool queryExit();

  /** Updates toolbar presentation after opening new document */
  void updateToolbar();

  /** Updates menus */
  void enableCommands();
  /** No descriptions */
  void createInterpolator(int type, double*x=0, double*y=0, int n=0);

  /** Write property of Preferences * pref */
  void set_preferences( Preferences * _newVal);
  /** Read property of Preferences * pref */
  Preferences * get_preferences();
  /** Apply changes done in preferences */
  void apply_preferences(void);


public slots:

  /** switch argument for Statusbar help entries on slot selection */
  void statusCallback(int id_);

  /** generate a new document in the actual view */
  void slotFileNew();
  /** open a document */
  void slotFileOpen();
  /** imports an image */
  void slotFileImportImage();
  /** save a document */
  void slotFileSave();
  /** save a document under a different filename*/
  void slotFileSaveAs();
  /** close the actual file */
  void slotFileClose();
  /** print the actual file */
  void slotFilePrint();
  /** exits the application */
  void slotFileQuit();
  /** put the marked text/object into the clipboard and remove
   * it from the document */
  void slotEditCut();
  /** put the marked text/object into the clipboard*/
  void slotEditCopy();
  /** paste the clipboard into the document*/
  void slotEditPaste();
  /** toggle the toolbar*/
  void slotViewToolBar();
  /** toggle the statusbar*/
  void slotViewStatusBar();
  /** sends new zoom to view*/
  void slotChangeZoom(int z);
  /** check to sort points in X */
  void slotSetSort();
  /** shows dialog to set active interpolator */
  void slotSetInterpolator();
  /** shows an about dlg*/
  void slotHelpAbout();
  /** change the status message of the whole statusbar temporary */
  void slotStatusHelpMsg(const QString &text);
  /**  Selects Curve Point mode */
  void slotSelCurvePoint();
  /**  Selects Axis Point mode */
  void slotSelAxisPoint();
  /** Selects normal Pointer mode */
  void slotSelPointer();
  /** Selects curve number c from combo box in toolbar */
  void slotSelCurve(int c);
  /** Selects axes number c from combo box in toolbar */
  void slotSelAxes(int c);

  /** Shows dialog for adding a new axes system to document */
  void slotNewAxes();
  /** Shows dialog to edit axes properties */
  void slotEditAxes();
  /** Shows dialog for adding a new curve to document */
  void slotNewCurve();
  /** Shows dialog to edit curves properties */
  void slotEditCurves();
  /** Shows dialog for adding a new grid to document */
  void slotNewGrid();
  /** Shows dialog to edit grids properties */
  void slotEditGrids();
  /** Shows dialog to edit export models for curves */
  void slotEditModels();
  /** Shows dialog to change user preferences */
  void slotPreferences();
  /** Shows dialog to create new plugin */
  void slotCreatePlugin();

  void slotRemoveCurveFromList(int);
  void slotRemoveAxesFromList(int);

  /** Replace Axes or Curve in the main toolbar lists */
  void slotReplaceAxes(QString s1, QString s2);
  void slotReplaceCurve(QString s1, QString s2);

  /** Shows dialog of setup for exporting */
  void slotFileExportSetup();
  /** Shows preview */
  void slotPreview();
  /** Export data */
  void slotFileExport();

  /** The actual exporting routine */
  void exportDoIt(QTextStream &stream);

  /** Calls view->slotRefresh to redraw everything */
  void slotRefresh();

  /** This function loads all the plugins defined by the user and stored in the specified directory */
  bool loadPlugins();
private: // Private methods

  void togglePointer(int id);
  bool generate_shared_library(const char *name);

signals: // Signals
  /** Emitted whenever the pointer mode changes */
  void changedPointerMode(int);

private:

  /** view is the main widget which represents your working area. The View
   * class should handle all events of the view widget.  It is kept empty so
   * you can create your view according to your application's needs by
   * changing the view class.
   */
  QScanPlotView *view;

  /** doc represents your actual document and is created only once. It keeps
   * information such as filename and does the serialization of your files.
   */
  QScanPlotDoc *doc;

  /** file_menu contains all items of the menubar entry "File" */
  QPopupMenu  *fileMenu;
  /** edit_menu contains all items of the menubar entry "Edit" */
  QPopupMenu  *editMenu;
  /** view_menu contains all items of the menubar entry "View" */
  QPopupMenu  *viewMenu;
  /** elem_menu contains all items of the menubar entry "Elements" */
  QPopupMenu  *elemMenu;
  /** opt_menu contains all items of the menubar entry "Options" */
  QPopupMenu  *optMenu;
  /** opt_menu contains all items of the menubar entry "Tools" */
  QPopupMenu  *toolMenu;
  /** help_menu contains all items of the menubar entry "Help" */
  QPopupMenu  *helpMenu;

  QToolBar    *fileToolbar;
  QToolButton *selPointer;
  QToolButton *selAxisPoint;
  QToolButton *selCurvePoint;

  QComboBox   *curvesList;
  QComboBox   *axesList;

  bool sortPoints;

  // variables to control the "Export" procedure

  dExportSetup * dialogExportSetup;

  bool exportFirstTime;

  typedef QValueList<int> ExportList;

  ExportList includedCurves;
  ExportList excludedCurves;

  int axesSystem2Use;
  int grid2Use;
  int exportPointsCriteria;
  int exportLayout;
  int exportSeparator;
  int exportComment;
  int exportNumeric;
  int exportSpecialFormat;

  QString exportSeparatorString;
  QString exportCommentString;
  QString exportUserFormatString;

  Interpolator* interpolator;
  int nInterpolator;

  /** Previous ZOOM selected in View menu */
  int oldz;

  /** Handle for plugins' shared libraries */
  void *hPlugins_Handle;

  /** User preferences */
  Preferences * pref;

protected slots: // Protected slots
  /** No descriptions */
  void slotAboutQt();
};
#endif 

