/*************************************************************************
 *                                                                          
 * qscanplotdoc.h  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/

#ifndef QSCANPLOTDOC_H
#define QSCANPLOTDOC_H

#include <math.h>

// include files for QT
#include <qobject.h>

// application specific includes

#include <qimage.h>
#include <qstring.h>
#include <qtextstream.h>
#include <qintdict.h>

#include "curve.h"
#include "axes.h"
#include "grid.h"
#include "qspparser.h"

inline void doScale(double *x, double *y, int typeX, int typeY)
{
  if (typeX == AXES_TYPE_LOGARITHMIC) { *x = log10(*x); }
  if (typeY == AXES_TYPE_LOGARITHMIC) { *y = log10(*y); }
}

inline void undoScale(double *x, double *y, int typeX, int typeY)
{
  if (typeX == AXES_TYPE_LOGARITHMIC) { *x = pow(10.0, *x); }
  if (typeY == AXES_TYPE_LOGARITHMIC) { *y = pow(10.0, *y); }
}

#define SERIALIZE_VERSION  0.1

/**
  * the Document Class
  */

class QScanPlotDoc : public QObject
{
  Q_OBJECT

friend QTextStream& operator<<(QTextStream &, const QScanPlotDoc &);
//friend QTextStream& operator>>(QTextStream &, QScanPlotDoc &);

friend class QspParser;

public:
  QScanPlotDoc();
  ~QScanPlotDoc();
  void newDoc();
  bool save();
  bool saveAs(const QString &filename);
  bool load(const QString &filename);
  bool isModified() const;
  void setModified(bool);

  /** Sets the canvas used in view for this document */
  inline void setCanvas(QCanvas* c) { theCanvas = c; }
  /** Retunrns name of file */
  inline QString fileName() const { return theFileName; }

  /** Loads image file in any of the supported formats */
  bool loadImage(const char* filename);
  /** Returns the pointer to the current image */
  QImage* getImage();
  /** Returns pointer to i-th curve */
  Curve* getCurve(int i) const;
  /** Returns pointer to i-th axes */
  Axes*  getAxes(int i) const;
  /** Returns pointer to i-th axes */
  Grid*  getGrid(int i) const;
  /** Returns pointer to current curve */
  Curve* getCurrentCurve() const;
  /** Returns pointer to current axes */
  Axes*  getCurrentAxes() const;
  /** Returns number of Axes systems defined in current document */
  int getNAxes() const;
  /** Returns number of curves defined in current document */
  int getNCurves() const;
  /** Returns number of curves defined in current document */
  int getNGrids() const;
  /** Selects Axes system */
  void selectAxes(int a);
  /** Selects curve c */
  void selectCurve(int c);
  /** Adds new Curve to list */
  void newCurve(Curve* c);
  /** Adds new Axes system to list */
  void newAxes(Axes* a);
  /** Adds new Axes system to list */
  void newGrid(Grid* g);
  /** Removes Curve from list */
  void deleteCurve(int n);
  /** Removes Axes from list */
  void deleteAxes(int n);
  /** Removes Grid from list */
  void deleteGrid(int n);
  /** Changes old Axes dict. with new one, and deletes the old */
  void useBackupAxes(QIntDict<Axes> * dict);
  /** Changes old Curves dict. with new one, and deletes the old */
  void useBackupCurves(QIntDict<Curve> * dict);

  /** Projects data of Curve nC using Axes system nA */
  bool projectData(int nA, int nC);

signals:
  void documentChanged();
  void deletedAxes(int);
  void deletedCurve(int);

protected:
  bool modified;

private:
  QImage           image;         // the loaded image
  QString          theFileName;   // the name of the current file
  QString          imageName;     // the name of the image

  int              nCurves;
  int              currentCurve;
  QIntDict<Curve>  *theCurves;

  int              nAxes;
  int              currentAxes;
  QIntDict<Axes>   *theAxes;

  int              nGrids;
  QIntDict<Grid>   *theGrids;

  QCanvas          *theCanvas;

  QspParser        *xmlParser;
};

#endif
