/*************************************************************************
 *                                                                          
 * qscanplotview.cpp  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#include "qscanplotview.h"

#include <qpainter.h>
#include <qapplication.h>
#include <qmsgbox.h>

#include "resource.h"

#include "curvepoint.h"
#include "axispoint.h"

#include "daxispoint.h"
#include "dnewelement.h"

#include "util.h"

#include <iostream>

QScanPlotView::QScanPlotView(QWidget *parent, QScanPlotDoc *doc) : QCanvasView(0, parent)
{
  setBackgroundMode( PaletteBase );

  theCanvas = new QCanvas();
  theCanvas->setDoubleBuffering( true );
  
  setCanvas(theCanvas);

  theDoc = doc;
  doc->setCanvas(theCanvas);

  initPopupMenus();

  /** connect doc with the view*/
  connect(doc, SIGNAL(documentChanged()), this, SLOT(slotDocumentChanged()));
}

QScanPlotView::~QScanPlotView()
{
}

void QScanPlotView::initPopupMenus()
{
  ///////////////////////////////////////////////////////////////////
  // axisPointMenu

  axisPointMenu=new QPopupMenu();
  axisPointMenu->setCaption("Axis Point");
  axisPointMenu->insertItem(tr("&Edit point..."), this, SLOT(slotAPEdit()), 0, ID_AP_EDIT);
  axisPointMenu->insertItem(tr("&Delete point..."), this, SLOT(slotAPDelete()), 0, ID_AP_DELETE);
  axisPointMenu->insertSeparator();
  axisPointMenu->insertItem(tr("E&dit this point's axes"), this, SLOT(slotAPEditAxes()), 0, ID_AP_EDITAXES);
  axisPointMenu->insertItem(tr("De&lete this point's axes"), this, SLOT(slotAPDeleteAxes()), 0, ID_AP_DELETEAXES);

  ///////////////////////////////////////////////////////////////////
  // curvePointMenu

  curvePointMenu=new QPopupMenu();
  curvePointMenu->setCaption("Axis Point");
  //curvePointMenu->insertItem(tr("&Edit point..."), this, SLOT(slotCPEdit()), 0, ID_CP_EDIT);
  curvePointMenu->insertItem(tr("&Delete point..."), this, SLOT(slotCPDelete()), 0, ID_CP_DELETE);
  curvePointMenu->insertSeparator();
  curvePointMenu->insertItem(tr("E&dit this point's curve"), this, SLOT(slotCPEditCurve()), 0, ID_CP_EDITCURVE);
  curvePointMenu->insertItem(tr("De&lete this point's curve"), this, SLOT(slotCPDeleteCurve()), 0, ID_CP_DELETECURVE);
}

void QScanPlotView::slotAPEdit()
{
  Axes *a = 0;
  for (int i=0; i<theDoc->getNAxes(); i++) {
    a = theDoc->getAxes(i);
    if ( a->contains(axisPoint) ) break;
    a = 0;
  }

  if (a == 0)
    qFatal("Something strange happens here. Couldn't find this axes.");

  dAxisPoint dlg;
  dlg.setName(a->getName());
  dlg.setDesc(a->getDesc());
  dlg.setGX( axisPoint->getGX() );
  dlg.setGY( axisPoint->getGY() );
  dlg.setX ( axisPoint->getX()  );
  dlg.setY ( axisPoint->getY()  );
  if ( dlg.exec() ) {
    axisPoint->setX ( dlg.getX()  );
    axisPoint->setY ( dlg.getY()  );
    axisPoint->setGX( dlg.getGX() );
    axisPoint->setGY( dlg.getGY() );
  }
}

void QScanPlotView::slotAPDelete()
{
  if ( QMessageBox::warning(0, "Deleting an Axis Point",
         "This will remove permanently from the Axes System the clicked point.\n"
         "Do you really want to remove this point?",
         QMessageBox::Yes,
         QMessageBox::No | QMessageBox::Default | QMessageBox::Escape )
       != QMessageBox::Yes )
    return;

  int i;
  for (i=0; i<theDoc->getNAxes(); i++) {
    if ( theDoc->getAxes(i)->contains(axisPoint) ) {
      theDoc->getAxes(i)->removePoint(axisPoint);
      return;
    }
  }
}

void QScanPlotView::slotAPEditAxes()
{
  dNewElement dlg;

  Axes *a = 0;
  for (int i=0; i<theDoc->getNAxes(); i++) {
    a = theDoc->getAxes(i);
    if ( a->contains(axisPoint) ) break;
    a = 0;
  }

  if (a == 0)
    qFatal("Something strange happens here. Couldn't find this axes.");

  QString name(a->getName());
  QString desc(a->getDesc());
  QColor  col(a->getColor());

  dlg.setTitle("Edit Axes System");
  dlg.setBtnColor(col);
  dlg.setName(name);
  dlg.setDesc(desc);
  dlg.setScales(a->getXAxisScale(), a->getYAxisScale());
  if ( dlg.exec() ) {
    QString oldname=name;
    name = dlg.getName();
    desc = dlg.getDesc();
    col  = dlg.getColor();
    a->setName( name );
    a->setDesc( desc );
    a->setColor( col );
    a->setScales(dlg.getXAxisScale(), dlg.getYAxisScale());
    a->propagateColor();
    if (oldname != name)
      emit replaceAxes(oldname, name);
  }
}

void QScanPlotView::slotAPDeleteAxes()
{
}

void QScanPlotView::slotCPEdit()
{
}

void QScanPlotView::slotCPDelete()
{
  if ( QMessageBox::warning(0, "Deleting a Curve Point",
         "This will remove permanently from the Curve the clicked point.\n"
         "Do you really want to remove this point?",
         QMessageBox::Yes,
         QMessageBox::No | QMessageBox::Default | QMessageBox::Escape )
       != QMessageBox::Yes )
    return;

  int i;
  for (i=0; i<theDoc->getNCurves(); i++) {
    if ( theDoc->getCurve(i)->contains(curvePoint) ) {
      theDoc->getCurve(i)->removePoint(curvePoint);
      return;
    }
  }
}

void QScanPlotView::slotCPEditCurve()
{
  dNewElement dlg;

  Curve *c;
  for (int i=0; i<theDoc->getNCurves(); i++) {
    c = theDoc->getCurve(i);
    if ( c->contains(curvePoint) ) break;
    c = 0;
  }

  if (c == 0)
    qFatal("Something strange happens here. Couldn't find this curve.");

  QString name(c->getName());
  QString desc(c->getDesc());
  QColor  col(c->getColor());

  dlg.setTitle("Edit Curve");
  dlg.setBtnColor(col);
  dlg.setName(name);
  dlg.setDesc(desc);
  dlg.hideAxesScales();
  if ( dlg.exec() ) {
    QString oldname=name;
    name = dlg.getName();
    desc = dlg.getDesc();
    col  = dlg.getColor();
    c->setName( name );
    c->setDesc( desc );
    c->setColor( col );
    c->propagateColor();
    if (oldname != name)
      emit replaceCurve(oldname, name);
  }
}

void QScanPlotView::slotCPDeleteCurve()
{
  if ( QMessageBox::critical(0, "Deleting Curve...",
         "Are you really sure you want to delete this curve,\nwith all its points?\n"
         "This will remove *permanently* the Curve with all the information.\n\n"
         "No, no, really, think about it.  Take your time...",
         QMessageBox::Yes,
         QMessageBox::No | QMessageBox::Default | QMessageBox::Escape)
      != QMessageBox::Yes )
    return;

  int i;
  for (i=0; i<theDoc->getNCurves(); i++)
    if ( theDoc->getCurve(i)->contains(curvePoint) ) break;

  if (i < theDoc->getNCurves()) theDoc->deleteCurve(i);
}

void QScanPlotView::slotDocumentChanged()
{
  image = theDoc->getImage();
  if (! image->isNull()) {
    convertImage();
    showImage();
    slotRefresh();
  } else {
    QPixmap nullpixmap;
    QCanvasItemList l=canvas()->allItems();
    for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
      delete *it;
    theCanvas->resize(0,0);
    theCanvas->setBackgroundPixmap(nullpixmap);
    refreshCanvas();
    repaintContents(frameRect(), TRUE);
    frameChanged();
    update();
  }
}

/** converts image to pixmap */
bool QScanPlotView::convertImage()
{
  bool success = false;

  if ( image->isNull() ) return false;

  QApplication::setOverrideCursor( waitCursor ); // this might take time

  if ( pm.convertFromImage(*image, PreferDither) ) {
    pmScaled = QPixmap();
    scale();
    //resize( width(), height() );
    success = true;                         // load successful
  } else {
    pm.resize(0,0);                         // couldn't load image
  }

  QApplication::restoreOverrideCursor();    // restore original cursor

  return success;                           // TRUE if loaded OK

}

/** scales pixmap to fit window */
void QScanPlotView::scale()
{
  int h = height();

  if ( image->isNull() ) return;

  QApplication::setOverrideCursor( waitCursor ); // this might take time

  if ( width() == pm.width() && h == pm.height() ) {
    pmScaled = pm;
  } else {
    //if (smooth()) {
    //  pmScaled.convertFromImage(image.smoothScale(width(), h), conversion_flags);
    //} else {
      QWMatrix m;                         // transformation matrix
      //m.scale(double(zoom*width())/pm.width(),// define scale factors
      //        double(zoom*height())/pm.height());
      m.scale(zoom, zoom);
      pmScaled = pm.xForm( m );           // create scaled pixmap
    //}
  }

  QApplication::restoreOverrideCursor();      // restore original cursor
}

/** Changes the zoom to view image */
void QScanPlotView::setInitialZoom(double z)
{
  zoom = z;
}

/** Changes the zoom to view image */
void QScanPlotView::setZoom(double z)
{
  zoom = z;
  scale();
  showImage();
  slotRefresh();
}

/** shows the scaled pixmap */
void QScanPlotView::showImage()
{
  if ( pm.size() != QSize( 0, 0 ) ) {
    theCanvas->setBackgroundPixmap(pmScaled);
    theCanvas->resize(pmScaled.width(), pmScaled.height());
    viewport()->setMaximumSize(pmScaled.size());
    frameChanged();
  }
}

void QScanPlotView::contentsMousePressEvent(QMouseEvent* e)
{
  /*
  qDebug("%% (%f, %f)",
         double(e->pos().x()) / pmScaled.width(),
         double(e->pos().y()) / pmScaled.height());
  */
  moving = 0;

  switch (e->button())
  {
    case LeftButton:
    {

      switch (pointerMode)
      {
        case ID_ON_POINTER:
        {
          break;
        }
        case ID_ON_AXISPOINT:
        {
          if (theDoc->getNAxes() < 1) {
            QMessageBox::information(this, "Invalid action",
                                     "You must create first at least one axes system.",
                                     QMessageBox::Ok);
            break;
          }

          if ( theDoc->getCurrentAxes()->complete() ) {
            QMessageBox::information(this, "Axes System completly defined",
                                     "The current Axes System [" +
                                     theDoc->getCurrentAxes()->getName() +
                                     "]\nalready has three axis points defined.",
                                     QMessageBox::Ok);
            break;
          }

          dAxisPoint dlg;
          dlg.setName(theDoc->getCurrentAxes()->getName());
          dlg.setDesc(theDoc->getCurrentAxes()->getDesc());
          dlg.setGX(double(e->pos().x()) / pmScaled.width());
          dlg.setGY(double(e->pos().y()) / pmScaled.height());

          if ( dlg.exec() ) {
            double x, y, gx, gy;
            x  = dlg.getX();
            y  = dlg.getY();
            gx = dlg.getGX();
            gy = dlg.getGY();
            //qDebug("Current axes ptr: %p", theDoc->getCurrentAxes());
            theDoc->getCurrentAxes()->newPoint(x, y, gx, gy);
          }	

          break;
        }
        case ID_ON_CURVEPOINT:
        {
          if (theDoc->getNCurves() < 1) {
            QMessageBox::information(this, "Invalid action",
                                     "You must declare first at least one curve "
                                     "to assign this point to.",
                                     QMessageBox::Ok);
            break;
          }

          //qDebug("Current curve ptr: %p", theDoc->getCurrentCurve());
          theDoc->getCurrentCurve()->newPoint(0., 0.,
                                              double(e->pos().x()) / pmScaled.width(),
                                              double(e->pos().y()) / pmScaled.height());
          redrawCurrentCurve();

          break;
        }
        default:
        {
        }
      }

    }
    case MidButton:
    {
      QCanvasItemList l=canvas()->collisions(e->pos());
      for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it) {
        if ( (*it)->rtti() & FigurePointRTTI ) {
          //qDebug("%% OVER A POINT!!");
    	    CurvePoint *item = (CurvePoint*)(*it);
          if ( ! (item->boundingRect()).contains( e->pos() ) ) continue;
        }
        moving = *it;
        moving_start = e->pos();
        return;
      }

      break;
    }
    case RightButton:
    {
      int rttiCode = 0;
      bool mustContinue = true;

      QCanvasItemList l=canvas()->collisions(e->pos());
      for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it) {
        rttiCode = (*it)->rtti();
        switch (rttiCode) {
        case AxisPointRTTI:
          axisPoint = (AxisPoint*)(*it);
          if ( ! (axisPoint->boundingRect()).contains( e->pos() ) ) {
            axisPoint = 0;
            mustContinue = true;
          } else {
            mustContinue = false;
          }
          break;
        case FigurePointRTTI:
          curvePoint = (CurvePoint*)(*it);
          if ( ! (curvePoint->boundingRect()).contains( e->pos() ) ) {
            curvePoint = 0;
            mustContinue = true;
          } else {
            mustContinue = false;
          }
          break;
        default:
          break;
        }
        if ( mustContinue ) continue;
      }

      switch (rttiCode) {

      case AxisPointRTTI:

        /* shows popup menu */
        axisPointMenu->exec(QCursor::pos());
        break;

      case FigurePointRTTI:

        /* shows popup menu */
        curvePointMenu->exec(QCursor::pos());
        break;

      default:

        /* shows popup menu */
        //imageAreaMenu->exec(QCursor::pos());
        break;

      }

      break;
    }
    default:
    {
    }
  }
  
  slotRefresh();
}

void QScanPlotView::contentsMouseMoveEvent(QMouseEvent* e)
{
  switch (e->button())
  {
    case LeftButton:
    {
      break;
    }
    case MidButton:
    {
      if ( moving ) {
        moving->moveBy(e->pos().x() - moving_start.x(),
                       e->pos().y() - moving_start.y());
        //canvas()->setChanged(QRect(min(moving_start.x(), e->pos().x())-10,
        //                           min(moving_start.y(), e->pos().y())-10,
        //                           abs(moving_start.x() - e->pos().x())-20,
        //                           abs(moving_start.y() - e->pos().y())-20));
        moving_start = e->pos();

        refreshCanvas();
      }

      break;
    }
    case RightButton:
    {
      break;
    }
    default:
    {
    }
  }
}

void QScanPlotView::contentsMouseReleaseEvent(QMouseEvent* e)
{
  switch (e->button())
  {
    case LeftButton:
    {
      break;
    }
    case MidButton:
    {
      if (moving != 0) {
   	    FigurePoint *item = (FigurePoint*)(moving);
        item->setGX(double(e->pos().x()) / pmScaled.width());
        item->setGY(double(e->pos().y()) / pmScaled.height());
      }

      break;
    }
    case RightButton:
    {
      break;
    }
    default:
    {
    }
  }

  moving = 0;

  slotRefresh();
}

/** Notifies the view that the pointer mode changed */
void QScanPlotView::slotChangedPointerMode(int m)
{
  //qDebug("%d -> %d", pointerMode, m);
  pointerMode = m;
}

/** Re-draw all axis points from all axes */
void QScanPlotView::redrawAxes()
{
  AxisPoint* p;
  int k, i, px, py;
  for (k=0; k<theDoc->getNAxes(); k++) {
    for (i=0; i<3; i++) {
      p = theDoc->getAxes(k)->getPoint(i);
      if (p != 0) {
        px = int(p->getGX()*pmScaled.width());
        py = int(p->getGY()*pmScaled.height());
        //qDebug("Point %d to show {%d:%p} is at (%d,%d)", i, k, p, px, py);
        p->move(px, py);
        p->show();
      }
    }
  }
}

/** Re-draw current curve */
void QScanPlotView::redrawCurrentCurve()
{
  redrawCurve( theDoc->getCurrentCurve() );
}

/** Re-draw all curves */
void QScanPlotView::redrawAllCurves()
{
  int i;
  for (i=0; i<theDoc->getNCurves(); i++)
    redrawCurve( theDoc->getCurve(i) );
}

/** Re-draw a given curve */
void QScanPlotView::redrawCurve(Curve* c)
{
  int px, py;

  CurvePoint *p;
  PointListIterator it(*(c->getPointList())); // iterator for list

  for ( ; it.current(); ++it ) {
    p = it.current();
    px = int(p->getGX()*pmScaled.width());
    py = int(p->getGY()*pmScaled.height());
    //qDebug("Point to show {%p} is at (%d,%d)", p, px, py);
    p->move(px, py);
    p->show();
  }
}

/** Redraws all */
void QScanPlotView::slotRefresh()
{
  redrawAxes();
  redrawAllCurves();
  refreshCanvas();
}

/** Redraw all */
void QScanPlotView::refreshCanvas()
{
  canvas()->setAllChanged();
  canvas()->update();
}

/** returns weighted mean position in the input column,
 *  in a neighbourhood of the input point
 */

