/*************************************************************************
 *                                                                          
 * qscanplotview.h  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#ifndef QSCANPLOTVIEW_H
#define QSCANPLOTVIEW_H

// include files for QT
#include <qcanvas.h>
#include <qpixmap.h>
#include <qpopupmenu.h>

// application specific includes
#include "qscanplotdoc.h"

class AxisPoint;
class FigurePoint;

/**
 * This class provides an incomplete base for your application view. 
 */

class QScanPlotView : public QCanvasView
{
  Q_OBJECT

public:
  QScanPlotView(QWidget *parent=0, QScanPlotDoc* doc=0);
  ~QScanPlotView();

  /** initialise popup menus */
  void initPopupMenus();

  /** converts image to pixmap */
  bool convertImage();
  /** scales pixmap to fit window */
  void scale();
  /** Changes the first zoom to view image */
  void setInitialZoom(double z);
  /** Changes the zoom to view image */
  void setZoom(double z);
  /** shows the scaled pixmap */
  void showImage();
  /** Re-draw all axis points from all axes */
  void redrawAxes();
  /** Re-draw current curve */
  void redrawCurrentCurve();
  /** Re-draw a given curve */
  void redrawCurve(Curve *c);
  /** Re-draw all curves */
  void redrawAllCurves();
  /** Update all views of canvas */
  void refreshCanvas();

protected:
  void contentsMousePressEvent(QMouseEvent*);
  void contentsMouseMoveEvent(QMouseEvent*);
  void contentsMouseReleaseEvent(QMouseEvent*);

signals:
  void replaceAxes(QString,QString);
  void replaceCurve(QString,QString);

public slots: // Public slots
  /** Notifies the view that the pointer mode changed */
  void slotChangedPointerMode(int);
  /** Notifies the view that the document changed somehow */
  void slotDocumentChanged();
  /** Redraw all */
  void slotRefresh();

  /* Slots related to Axis Point Popup Menu */
  void slotAPEdit();
  void slotAPDelete();
  void slotAPEditAxes();
  void slotAPDeleteAxes();

  /* Slots related to Curve Point Popup Menu */
  void slotCPEdit();
  void slotCPDelete();
  void slotCPEditCurve();
  void slotCPDeleteCurve();

private:
  QScanPlotDoc *theDoc;

  QPopupMenu   *axisPointMenu;
  QPopupMenu   *curvePointMenu;
  QPopupMenu   *imageAreaMenu;

  QImage       *image;
  QPixmap      pm;            // the converted pixmap
  QPixmap      pmScaled;      // the scaled pixmap
  double       zoom;
  QCanvas      *theCanvas;

  QCanvasItem* moving;
  QPoint       moving_start;	

  AxisPoint   * axisPoint;
  CurvePoint  * curvePoint;

  int          pointerMode;
};

#endif
