/*************************************************************************
 *                                                                          
 * qspparser.cpp  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#include "qspparser.h"

#include "qscanplotdoc.h"
#include <qstring.h>
#include <qmap.h>
#include <qmessagebox.h>

#include "plugin.h"

extern QMap <QString, Builder_ptr> Plugins_Factory;

QspParser::QspParser() {}

QspParser::~QspParser() {}

bool QspParser::startDocument()
{
  indent = "";
  state = StateInit;
  return TRUE;
}

bool QspParser::startElement( const QString&, const QString&,
                              const QString& qName, const QXmlAttributes& atts)
{
  cout << indent << qName;

  int i;
  int n = atts.length();

  if (n > 0) {
    cout << " : { ";
    for (i=0; i<n-1; i++)
      cout << atts.qName(i) << " = " << atts.value(i) << ", ";
    cout << atts.qName(i) << " = " << atts.value(i) << " }";
  }

  cout << endl << flush;

  QColor  col;
  QString name;
  QString desc;
  int index;
  int model;
  int fx, fy;

  if ((qName == "qscanplot") && (state == StateInit)) {

    state = StateDocument;
    nA = nC = 0;

  } else if ((qName == "image") && (state == StateDocument)) {

    state = StateImage;
    theDoc->imageName = atts.value("src");

  } else if ((qName == "axessystems") && (state == StateDocument)) {

    state = StateAxesSystems;
    nAxes = atts.value("number").toInt();

  } else if ((qName == "curves") && (state == StateDocument)) {

    state = StateCurves;
    nCurves = atts.value("number").toInt();

  } else if ((qName == "grids") && (state == StateDocument)) {

    state = StateGrids;
    nGrids = atts.value("number").toInt();

  } else if ((qName == "plugins") && (state == StateDocument)) {

    state = StatePlugins;
    nPlugins = atts.value("number").toInt();

  } else if ((qName == "axes") && (state == StateAxesSystems)) {

    state = StateAxes;
    a = new Axes;
    name = atts.value("name");
    desc = atts.value("description");
    col = QColor(atts.value("color"));
    a->setName( name );
    a->setDesc( desc );
    a->setScales( atts.value("xscale").toInt(), atts.value("yscale").toInt() );
    a->setColor( col );
    theDoc->newAxes( a );
    nA++;

  } else if ((qName == "curve") && (state == StateCurves)) {

    state = StateCurve;
    c = new Curve;
    name = atts.value("name");
    desc = atts.value("description");
    model = atts.value("model").toInt();
    fx = atts.value("fx").toInt();
    fy = atts.value("fy").toInt();
    col = QColor(atts.value("color"));
    c->setName( name );
    c->setDesc( desc );
    c->setColor( col );

    Model mdl = c->getModel();

    if ( Plugins_Factory.contains( listPlugins[fx] ) || (fx == 0) ) {
      mdl.setFx( fx );
      cout << "using " << listPlugins[fx] << endl;
    } else {
      mdl.setFx( 0 );
      model = model & (~ MODEL_FX_Y);
      QMessageBox::information(0, "Plug-in not found",
                               QString("This document contains a curve with a reference to a\n") +
                               QString("plug-in module that is not loaded (") +
                               listPlugins[fx] +
                               QString(").\nThe reference will be removed."),
                               QMessageBox::Ok);
    }

    if ( Plugins_Factory.contains( listPlugins[fy] ) || (fy == 0) ) {
      mdl.setFy( fy );
      cout << "using " << listPlugins[fy] << endl;
    } else {
      mdl.setFy( 0 );
      model = model & (~ MODEL_X_GY);
      QMessageBox::information(0, "Plug-in not found",
                               QString("This document contains a curve with a reference to a\n") +
                               QString("plug-in module that is not loaded (") +
                               listPlugins[fy] +
                               QString(").\nThe reference will be removed."),
                               QMessageBox::Ok);
    }

    mdl.setModel( model );

    theDoc->newCurve( c );
    nC++;

  } else if ((qName == "grid") && (state == StateGrids)) {

    state = StateGrid;
    g = new Grid;
    name = atts.value("name");
    desc = atts.value("description");
    g->setName( name );
    g->setDesc( desc );
    g->init( atts.value("xmin").toDouble(),
             atts.value("xmax").toDouble(),
             atts.value("intervals").toInt() );
    theDoc->newGrid( g );
    nG++;

  } else if ((qName == "plugin") && (state == StatePlugins)) {

    state = StatePlugin;
    name  = atts.value("name");
    index = atts.value("index").toInt();
    listPlugins[index] = name;
    cout << "adding " << name << " to list." << endl;
    nP++;

  } else if ((qName == "point") && (state == StateAxes)) {

    state = StateAxisPoint;
    a->newPoint(atts.value("x").toDouble(),  atts.value("y").toDouble(),
                atts.value("gx").toDouble(), atts.value("gy").toDouble());

  } else if ((qName == "point") && (state == StateCurve)) {

    state = StateCurvePoint;
    c->newPoint(0., 0.,
                atts.value("gx").toDouble(), atts.value("gy").toDouble());

  } else {

    return FALSE; // error in format!

  }

  indent += "  ";
  return TRUE;
}

bool QspParser::endElement( const QString&, const QString&, const QString& )
{
  // "pop" the state (and perhaps do some actions)
  switch (state) {

  case StateImage:
  case StateAxesSystems:
  case StateCurves:
  case StateGrids:
  case StatePlugins:

    state = StateDocument;
    break;

  case StateAxes:

    state = StateAxesSystems;
    break;

  case StateCurve:

    state = StateCurves;
    break;

  case StateGrid:

    state = StateGrids;
    break;

  case StatePlugin:

    state = StatePlugins;
    break;

  case StateAxisPoint:

    state = StateAxes;
    break;

  case StateCurvePoint:

    state = StateCurve;
    break;

  default:

    break;    // do nothing

  }

  indent.remove( 0, 2 );
  return TRUE;
}

QString QspParser::errorProtocol()
{
  return errorProt;
}

QString QspParser::errorString()
{
  return "The document is not in the QScanPlot (qscanplot) file format";
}

bool QspParser::fatalError(const QXmlParseException & exception)
{
  errorProt += QString("fatal parsing error: %1 in line %2, column %3\n")
                      .arg (exception.message())
                      .arg (exception.lineNumber())
                      .arg (exception.columnNumber());

  return QXmlDefaultHandler::fatalError(exception);
}

/** Sets the doc to be used */
void QspParser::setDoc(QScanPlotDoc* doc)
{
  theDoc = doc;
}
