/*************************************************************************
 *                                                                          
 * qspparser.h  -  description
 *
 * Copyright (C) 2001  J C Gonzalez
 * gonzalez@gae.ucm.es
 *
 *------------------------------------------------------------------------
 *
 * Copyright (C) 2001 J C Gonzalez
 *  
 * This program is free software;  you can redistribute it and/or  modify
 * it under the terms  of the GNU General  Public License as published by
 * the Free Software Foundation; either version  2 of the License, or (at
 * your option) any later version.
 * 
 * This piece of code is distributed in the hope  that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * In no  event shall his author  be liable for  any special, incidental,
 * indirect  or  consequential  damages  of any  kind,   or  any  damages
 * whatsoever resulting from loss of use, data or profits, whether or not
 * advised of the possibility of damage, and on  any theory of liability,
 * arising out  of or in connection  with the use  or performance of this
 * software. You've been warned.
 * 
 ************************************************************************/


#ifndef QSPPARSER_H
#define QSPPARSER_H

#include <iostream.h>

#include <qxml.h>
#include <qintdict.h>
#include <qcanvas.h>
#include <qmap.h>
#include <qstring.h>

/**XML Parser for reading .qsp documents
  *@author J C Gonzalez
  */

class QScanPlotDoc;
class Axes;
class Curve;
class Grid;

class QspParser : public QXmlDefaultHandler  {
public: 
	QspParser();
	~QspParser();

public:
  bool startDocument();
  bool startElement( const QString&, const QString&,
                     const QString& , const QXmlAttributes& );
  bool endElement( const QString&, const QString&, const QString& );
  bool fatalError(const QXmlParseException & exception);
  QString errorString();
  QString errorProtocol ();

  /** Sets the doc to be used */
  void setDoc(QScanPlotDoc*);

private:
  QString indent;
  QString errorProt;

  enum State {
    StateInit,
    StateDocument,
    StateImage,
    StateAxesSystems,
    StateAxes,
    StateCurves,
    StateCurve,
    StateGrids,
    StateGrid,
    StatePlugins,
    StatePlugin,
    StateAxisPoint,
    StateCurvePoint
  };

  State state;

  QScanPlotDoc  *theDoc;
  Axes          *a;
  Curve         *c;
  Grid          *g;
  int           nAxes, nCurves, nGrids, nPlugins;
  int           nA, nC, nG, nP;
  QMap <int, QString> listPlugins;

};

#endif
